import numpy as np
from ..affine import power_L

def basil_inner_loop(path_obj,
                     lagrange_subseq,
                     initial_data, # (solution, grad) pair
                     inner_tol=1.e-5,
                     verbose=False,
                     initial_step=None,
                     check_active=False,
                     step_nvar=50,
                     candidate_set=None,
                     solve_args={}):

    debug = True
    coef_stop = False

    # take a guess at the inverse step size
    if initial_step is None:
        _lipschitz = power_L(path_obj.X, max_its=50)
        final_step = 1000. / _lipschitz 
    else:
        final_step = initial_step

    # gradient of restricted elastic net at lambda_max

    solution, grad_solution = initial_data

    solution = solution.copy()
    last_grad = grad_solution.copy()
    inactive_ranks = path_obj.check_KKT(grad_solution,
                                        solution,
                                        path_obj.alpha * np.min(lagrange_subseq))[1]

    linear_predictor = path_obj.X.dot(solution)
    unpen = np.zeros(path_obj.group_shape, np.bool)
    unpen[path_obj.unpenalized[1]] = True
    ever_active = list(path_obj.updated_ever_active(unpen)) # unpenalized groups

    solutions = []
    lagrange_solved = []

    all_failing = np.zeros(path_obj.group_shape, np.bool)

    M = min(step_nvar, (inactive_ranks >= 0).sum())

    candidate_bool = np.zeros(path_obj.group_shape, np.bool)
    subproblem_set = path_obj.updated_ever_active((inactive_ranks < M) * (inactive_ranks >= 0) +
                                                  unpen)
    if candidate_set is not None:
        subproblem_set = sorted(set(subproblem_set + candidate_set))

    for lagrange in lagrange_subseq:

        (final_step, 
         subproblem_grad, 
         subproblem_soln,
         subproblem_linpred,
         subproblem_vars) = path_obj.solve_subproblem(solution, # for warm start
                                                      subproblem_set,
                                                      lagrange,
                                                      tol=inner_tol,
                                                      start_step=final_step,
                                                      debug=debug and verbose,
                                                      coef_stop=coef_stop,
                                                      **solve_args)

        saturated_grad = path_obj.saturated_loss.smooth_objective(subproblem_linpred, 'grad')
        # as subproblem always contains ever active, 
        # rest of solution should be 0
        solution[subproblem_vars] = subproblem_soln

        enet_grad = path_obj.enet_grad(solution, 
                                       path_obj._penalized_vars,
                                       lagrange)
        grad_solution[:] = (path_obj.full_gradient(path_obj.saturated_loss, 
                                                   subproblem_linpred) + 
                            enet_grad)
        failing_A, failing_I_ranks = path_obj.check_KKT(grad_solution, 
                                                        solution, 
                                                        path_obj.alpha * lagrange)

        if check_active:
            all_failing = failing_A + (failing_I_ranks >= 0)
        else:
            all_failing = failing_I_ranks >= 0

        if not all_failing.sum():
            ever_active_incr = path_obj.updated_ever_active(path_obj.active_set(solution))
            ever_active = sorted(set(ever_active + ever_active_incr))
            linear_predictor[:] = subproblem_linpred
            solutions.append(solution.T.copy())
            last_grad = grad_solution.copy()
            lagrange_solved.append(lagrange)
        else:
            ever_active1 = ever_active + list(subproblem_set)
            ever_active2 = path_obj.updated_ever_active(all_failing) 
            ever_active = sorted(set(ever_active1 + ever_active2))
            break

    return (lagrange_solved, 
            solutions, 
            last_grad,
            ever_active)

def basil(path_obj,
          lagrange_seq,
          initial_data, # (solution, grad) pair
          inner_tol=1.e-5,
          verbose=False,
          initial_step=None,
          check_active=False,
          step_nvar=50,
          step_lagrange=5,
          solve_args={}):

    lagrange_solved, solutions, candidate_set = [np.inf], [], []

    cur_data = initial_data

    while True:
        
        lagrange_start = np.sum(lagrange_seq >= np.min(lagrange_solved))
        lagrange_cand = lagrange_seq[lagrange_start:(lagrange_start + step_lagrange)] 

        if len(lagrange_cand) > 0:
            (lagrange_incr, 
             solution_incr, 
             last_grad,
             candidate_set) = basil_inner_loop(path_obj,
                                               lagrange_cand,
                                               cur_data,
                                               inner_tol=inner_tol,
                                               step_nvar=step_nvar,
                                               candidate_set=candidate_set,
                                               solve_args=solve_args)
            if len(solution_incr) > 0:
                cur_soln = (solution_incr[-1], last_grad)

                solutions.extend(solution_incr)
                lagrange_solved.extend(lagrange_incr)
        else:
            break

    return np.array(solutions)
        



from itertools import product
import nose.tools as nt

import numpy as np
import scipy.sparse

import regreg.api as rr
import regreg.affine.image2d as I2D
from regreg.identity_quadratic import identity_quadratic as sq
from regreg.tests.decorators import set_seed_for_test

@set_seed_for_test()
def test_class():
    p = 50
    diff = I2D.image2d_differences((p,p+5))
    diff2 = I2D.image2d_differences((p,p+5), affine_offset=1)
    im = np.random.standard_normal((p,p+5))
    di = np.random.standard_normal(diff.output_shape)
    diff.linear_map(im)
    diff.affine_map(im)
    diff.adjoint_map(di)

    I2D.formD_smaller(p, p+5)

    np.testing.assert_allclose(diff2.affine_map(im), diff2.linear_map(im) + 1)



from itertools import product
import nose.tools as nt

import numpy as np
import scipy.sparse

import regreg.api as rr
from regreg.identity_quadratic import identity_quadratic as sq
from regreg.tests.decorators import set_seed_for_test

@set_seed_for_test()
def test_centering():
    """
    This test verifies that the normalized transform
    of affine correctly implements the linear
    transform that multiplies first by X, then centers.
    """
    # N - number of data points
    # P - number of columns in design == number of betas
    N, P = 40, 30
    # an arbitrary positive offset for data and design
    offset = 50

    # design - with ones as last column
    X = np.ones((N,P))
    X[:,:-1] = np.random.normal(size=(N,P-1)) + offset
    X2 = X - X.mean(axis=0)[np.newaxis,:]
    L = rr.normalize(X, center=True, scale=False)
    # coef for loss

    for _ in range(10):
        beta = np.random.normal(size=(P,))
        v = L.linear_map(beta)
        v2 = np.dot(X, beta)
        v2 -= v2.mean()
        v3 = np.dot(X2, beta)
        v4 = L.affine_map(beta)
        np.testing.assert_almost_equal(v, v3)
        np.testing.assert_almost_equal(v, v2)
        np.testing.assert_almost_equal(v, v4)

        y = np.random.standard_normal(N)
        u1 = L.adjoint_map(y)
        y2 = y - y.mean()
        u2 = np.dot(X.T, y2)
        np.testing.assert_almost_equal(u1, u2)

@set_seed_for_test()
def test_scaling():
    """
    This test verifies that the normalized transform
    of affine correctly implements the linear
    transform that multiplies first by X, then centers.
    """
    # N - number of data points
    # P - number of columns in design == number of betas
    N, P = 40, 30
    # an arbitrary positive offset for data and design
    offset = 50

    # design - with ones as last column
    X = np.ones((N,P))
    X[:,:-1] = np.random.normal(size=(N,P-1)) + offset

    L = rr.normalize(X, center=False, scale=True)
    # coef for loss

    scalings = np.sqrt((X**2).sum(0) / N)
    scaling_matrix = np.diag(1./scalings)
    
    for _ in range(10):

        beta = np.random.normal(size=(P,))
        v = L.linear_map(beta)
        v2 = np.dot(X, np.dot(scaling_matrix, beta))
        v3 = L.affine_map(beta)
        np.testing.assert_almost_equal(v, v2)
        np.testing.assert_almost_equal(v, v3)

        y = np.random.standard_normal(N)
        u1 = L.adjoint_map(y)
        u2 = np.dot(scaling_matrix, np.dot(X.T, y))
        np.testing.assert_almost_equal(u1, u2)

@set_seed_for_test()
def test_scaling_and_centering():
    """
    This test verifies that the normalized transform
    of affine correctly implements the linear
    transform that multiplies first by X, then centers.
    """
    # N - number of data points
    # P - number of columns in design == number of betas
    N, P = 40, 30
    # an arbitrary positive offset for data and design
    offset = 50

    # design - with no colum of ones!
    X = np.random.normal(size=(N,P)) + offset

    L = rr.normalize(X, center=True, scale=True) # the default
    # coef for loss

    scalings = np.std(X, 0)
    scaling_matrix = np.diag(1./scalings)

    for _ in range(10):
        beta = np.random.normal(size=(P,))
        v = L.linear_map(beta)
        v2 = np.dot(X, np.dot(scaling_matrix, beta))
        v2 -= v2.mean()
        np.testing.assert_almost_equal(v, v2)

        y = np.random.standard_normal(N)
        u1 = L.adjoint_map(y)
        y2 = y - y.mean()
        u2 = np.dot(scaling_matrix, np.dot(X.T, y2))
        np.testing.assert_almost_equal(u1, u2)

@set_seed_for_test()
def test_centering_fit(debug=False):

    # N - number of data points
    # P - number of columns in design == number of betas
    N, P = 40, 30
    # an arbitrary positive offset for data and design
    offset = 50

    # design - with ones as last column
    X = np.ones((N,P))
    X = np.random.normal(size=(N,P)) + offset
    X2 = X - X.mean(axis=0)[np.newaxis,:]

    # the normalizer
    L = rr.normalize(X, center=True, scale=False)

    # data
    Y = np.random.normal(size=(N,)) + offset

    # coef for loss
    coef = 0.5
    # lagrange for penalty
    lagrange = .1

    # Loss function (squared difference between fitted and actual data)
    loss = rr.quadratic_loss.affine(L, -Y, coef=coef)

    penalties = [rr.constrained_positive_part(25, lagrange=lagrange),
                 rr.nonnegative(5)]
    groups = [slice(0,25), slice(25,30)]
    penalty = rr.separable((P,), penalties,
                           groups)
    initial = np.random.standard_normal(P)

    composite_form = rr.separable_problem.fromatom(penalty, loss)
    solver = rr.FISTA(composite_form)
    solver.debug = debug
    solver.fit(tol=1.0e-12, min_its=200)
    coefs = solver.composite.coefs

    # Solve the problem with X2
    loss2 = rr.quadratic_loss.affine(X2, -Y, coef=coef)

    initial2 = np.random.standard_normal(P)
    composite_form2 = rr.separable_problem.fromatom(penalty, loss2)

    for _ in range(10):
        beta = np.random.standard_normal(P)
        g1 = loss.smooth_objective(beta, mode='grad')
        g2 = loss2.smooth_objective(beta, mode='grad')
        np.testing.assert_almost_equal(g1, g2)
        b1 = penalty.proximal(sq(1, beta, g1, 0))
        b2 = penalty.proximal(sq(1, beta, g1, 0))
        np.testing.assert_almost_equal(b1, b2)

        f1 = composite_form.objective(beta)
        f2 = composite_form2.objective(beta)
        np.testing.assert_almost_equal(f1, f2)

    solver2 = rr.FISTA(composite_form2)
    solver2.debug = debug
    solver2.fit(tol=1.0e-12, min_its=200)
    coefs2 = solver2.composite.coefs

    np.testing.assert_almost_equal(composite_form.objective(coefs), composite_form.objective(coefs2))
    np.testing.assert_almost_equal(composite_form2.objective(coefs), composite_form2.objective(coefs2))

    nt.assert_true(np.linalg.norm(coefs - coefs2) / max(np.linalg.norm(coefs),1) < 1.0e-04)

@set_seed_for_test()
def test_scaling_fit(debug=False):

    # N - number of data points
    # P - number of columns in design == number of betas
    N, P = 40, 30
    # an arbitrary positive offset for data and design
    offset = 2

    # design - with ones as last column
    X = np.ones((N,P))
    X[:,:-1] = np.random.normal(size=(N,P-1)) + offset
    X2 = X / (np.sqrt((X**2).sum(0) / N))[np.newaxis,:]
    L = rr.normalize(X, center=False, scale=True)

    # data
    Y = np.random.normal(size=(N,)) + offset

    # coef for loss
    coef = 0.5
    # lagrange for penalty
    lagrange = .1

    # Loss function (squared difference between fitted and actual data)
    loss = rr.quadratic_loss.affine(L, -Y, coef=coef)

    penalties = [rr.constrained_positive_part(25, lagrange=lagrange),
                 rr.nonnegative(5)]
    groups = [slice(0,25), slice(25,30)]
    penalty = rr.separable((P,), penalties,
                           groups)
    initial = np.random.standard_normal(P)
    composite_form = rr.separable_problem.fromatom(penalty, loss)

    solver = rr.FISTA(composite_form)
    solver.debug = debug
    solver.fit(tol=1.0e-12, min_its=200)
    coefs = solver.composite.coefs

    # Solve the problem with X2
    loss2 = rr.quadratic_loss.affine(X2, -Y, coef=coef)

    initial2 = np.random.standard_normal(P)
    composite_form2 = rr.separable_problem.fromatom(penalty, loss2)

    solver2 = rr.FISTA(composite_form2)
    solver2.debug = debug
    solver2.fit(tol=1.0e-12, min_its=200)
    coefs2 = solver2.composite.coefs

    for _ in range(10):
        beta = np.random.standard_normal(P)
        g1 = loss.smooth_objective(beta, mode='grad')
        g2 = loss2.smooth_objective(beta, mode='grad')
        np.testing.assert_almost_equal(g1, g2)
        b1 = penalty.proximal(sq(1, beta, g1, 0))
        b2 = penalty.proximal(sq(1, beta, g2, 0))
        np.testing.assert_almost_equal(b1, b2)

        f1 = composite_form.objective(beta)
        f2 = composite_form2.objective(beta)
        np.testing.assert_almost_equal(f1, f2)


    np.testing.assert_almost_equal(composite_form.objective(coefs), composite_form.objective(coefs2))
    np.testing.assert_almost_equal(composite_form2.objective(coefs), composite_form2.objective(coefs2))

    nt.assert_true(np.linalg.norm(coefs - coefs2) / max(np.linalg.norm(coefs),1) < 1.0e-04)

@set_seed_for_test()
def test_scaling_and_centering_fit(debug=False):

    # N - number of data points
    # P - number of columns in design == number of betas
    N, P = 40, 30
    # an arbitrary positive offset for data and design
    offset = 2

    # design - with ones as last column
    X = np.random.normal(size=(N,P)) + offset
    X2 = X - X.mean(0)[np.newaxis,:]
    X2 = X2 / np.std(X2,0)[np.newaxis,:]

    L = rr.normalize(X, center=True, scale=True)
    # data
    Y = np.random.normal(size=(N,)) + offset

    # coef for loss
    coef = 0.5
    # lagrange for penalty
    lagrange = .1

    # Loss function (squared difference between fitted and actual data)
    loss = rr.quadratic_loss.affine(L, -Y, coef=coef)

    penalties = [rr.constrained_positive_part(25, lagrange=lagrange),
                 rr.nonnegative(5)]
    groups = [slice(0,25), slice(25,30)]
    penalty = rr.separable((P,), penalties,
                           groups)

    initial = np.random.standard_normal(P)
    composite_form = rr.separable_problem.fromatom(penalty, loss)
    solver = rr.FISTA(composite_form)
    solver.debug = debug
    solver.fit(tol=1.0e-12, min_its=200)
    coefs = solver.composite.coefs

    # Solve the problem with X2
    loss2 = rr.quadratic_loss.affine(X2, -Y, coef=coef)

    initial2 = np.random.standard_normal(P)
    composite_form2 = rr.separable_problem.fromatom(penalty, loss2)

    solver2 = rr.FISTA(composite_form2)
    solver2.debug = debug
    solver2.fit(tol=1.0e-12, min_its=200)
    coefs2 = solver2.composite.coefs

    for _ in range(10):
        beta = np.random.standard_normal(P)
        g1 = loss.smooth_objective(beta, mode='grad')
        g2 = loss2.smooth_objective(beta, mode='grad')
        np.testing.assert_almost_equal(g1, g2)
        b1 = penalty.proximal(sq(1, beta, g1, 0))
        b2 = penalty.proximal(sq(1, beta, g2, 0))
        np.testing.assert_almost_equal(b1, b2)

        f1 = composite_form.objective(beta)
        f2 = composite_form2.objective(beta)
        np.testing.assert_almost_equal(f1, f2)

    np.testing.assert_almost_equal(composite_form.objective(coefs), composite_form.objective(coefs2))
    np.testing.assert_almost_equal(composite_form2.objective(coefs), composite_form2.objective(coefs2))

    nt.assert_true(np.linalg.norm(coefs - coefs2) / max(np.linalg.norm(coefs),1) < 1.0e-04)

@set_seed_for_test()
def test_scaling_and_centering_fit_inplace(debug=False):

    # N - number of data points
    # P - number of columns in design == number of betas
    N, P = 40, 30
    # an arbitrary positive offset for data and design
    offset = 2

    # design
    X = np.random.normal(size=(N,P)) + offset
    L = rr.normalize(X, center=True, scale=True, inplace=True)

    # X should have been normalized in place
    np.testing.assert_almost_equal(np.sum(X**2, 0), N)
    np.testing.assert_almost_equal(np.sum(X, 0), 0)

    # data
    Y = np.random.normal(size=(N,)) + offset

    # coef for loss
    coef = 0.5
    # lagrange for penalty
    lagrange = .1

    # Loss function (squared difference between fitted and actual data)
    loss = rr.quadratic_loss.affine(L, -Y, coef=coef)

    penalties = [rr.constrained_positive_part(25, lagrange=lagrange),
                 rr.nonnegative(5)]
    groups = [slice(0,25), slice(25,30)]
    penalty = rr.separable((P,), penalties,
                           groups)

    initial = np.random.standard_normal(P)
    composite_form = rr.separable_problem.fromatom(penalty, loss)

    solver = rr.FISTA(composite_form)
    solver.debug = debug
    solver.fit(tol=1.0e-12, min_its=200)
    coefs = solver.composite.coefs

    # Solve the problem with X, which has been normalized in place
    loss2 = rr.quadratic_loss.affine(X, -Y, coef=coef)

    initial2 = np.random.standard_normal(P)
    composite_form2 = rr.separable_problem.fromatom(penalty, loss2)

    solver2 = rr.FISTA(composite_form2)
    solver2.debug = debug
    solver2.fit(tol=1.0e-12, min_its=200)
    coefs2 = solver2.composite.coefs

    for _ in range(10):
        beta = np.random.standard_normal(P)
        g1 = loss.smooth_objective(beta, mode='grad')
        g2 = loss2.smooth_objective(beta, mode='grad')
        np.testing.assert_almost_equal(g1, g2)
        b1 = penalty.proximal(sq(1, beta, g1, 0))
        b2 = penalty.proximal(sq(1, beta, g2, 0))
        np.testing.assert_almost_equal(b1, b2)

        f1 = composite_form.objective(beta)
        f2 = composite_form2.objective(beta)
        np.testing.assert_almost_equal(f1, f2)

    np.testing.assert_almost_equal(composite_form.objective(coefs), composite_form.objective(coefs2))
    np.testing.assert_almost_equal(composite_form2.objective(coefs), composite_form2.objective(coefs2))

    nt.assert_true(np.linalg.norm(coefs - coefs2) / max(np.linalg.norm(coefs),1) < 1.0e-04)

@set_seed_for_test()
def test_scaling_fit_inplace(debug=False):

    # N - number of data points
    # P - number of columns in design == number of betas
    N, P = 40, 30
    # an arbitrary positive offset for data and design
    offset = 2

    # design - with ones as last column
    X = np.ones((N,P))
    X[:,:-1] = np.random.normal(size=(N,P-1)) + offset
    L = rr.normalize(X, center=False, scale=True, inplace=True)

    # X should have been normalized in place
    np.testing.assert_almost_equal(np.sum(X**2, 0), N)

    # data
    Y = np.random.normal(size=(N,)) + offset

    # coef for loss
    coef = 0.5
    # lagrange for penalty
    lagrange = .1

    # Loss function (squared difference between fitted and actual data)
    loss = rr.quadratic_loss.affine(L, -Y, coef=coef)

    penalties = [rr.constrained_positive_part(25, lagrange=lagrange),
                 rr.nonnegative(5)]
    groups = [slice(0,25), slice(25,30)]
    penalty = rr.separable((P,), penalties,
                           groups)
    initial = np.random.standard_normal(P)
    composite_form = rr.separable_problem.fromatom(penalty, loss)

    solver = rr.FISTA(composite_form)
    solver.debug = debug
    solver.fit(tol=1.0e-12, min_its=200)
    coefs = solver.composite.coefs

    # Solve the problem with X, which has been normalized in place
    loss2 = rr.quadratic_loss.affine(X, -Y, coef=coef)

    initial2 = np.random.standard_normal(P)
    composite_form2 = rr.separable_problem.fromatom(penalty, loss2)

    solver2 = rr.FISTA(composite_form2)
    solver2.debug = debug
    solver2.fit(tol=1.0e-12, min_its=200)
    coefs2 = solver2.composite.coefs

    for _ in range(10):
        beta = np.random.standard_normal(P)
        g1 = loss.smooth_objective(beta, mode='grad')
        g2 = loss2.smooth_objective(beta, mode='grad')
        np.testing.assert_almost_equal(g1, g2)
        b1 = penalty.proximal(sq(1, beta, g1,0))
        b2 = penalty.proximal(sq(1, beta, g2,0))
        np.testing.assert_almost_equal(b1, b2)

        f1 = composite_form.objective(beta)
        f2 = composite_form2.objective(beta)
        np.testing.assert_almost_equal(f1, f2)


    np.testing.assert_almost_equal(composite_form.objective(coefs), composite_form.objective(coefs2))
    np.testing.assert_almost_equal(composite_form2.objective(coefs), composite_form2.objective(coefs2))

    nt.assert_true(np.linalg.norm(coefs - coefs2) / max(np.linalg.norm(coefs),1) < 1.0e-04)

@set_seed_for_test()
def test_centering_fit_inplace(debug=False):

    # N - number of data points
    # P - number of columns in design == number of betas
    N, P = 40, 30
    # an arbitrary positive offset for data and design
    offset = 2

    # design - with ones as last column
    X = np.random.normal(size=(N,P)) + offset
    L = rr.normalize(X, center=True, scale=False, inplace=True)

    # X should have been normalized in place
    np.testing.assert_almost_equal(np.sum(X, 0), 0)

    # data
    Y = np.random.normal(size=(N,)) + offset

    # coef for loss
    coef = 0.5
    # lagrange for penalty
    lagrange = .1

    # Loss function (squared difference between fitted and actual data)
    loss = rr.quadratic_loss.affine(L, -Y, coef=coef)

    penalties = [rr.constrained_positive_part(25, lagrange=lagrange),
                 rr.nonnegative(5)]
    groups = [slice(0,25), slice(25,30)]
    penalty = rr.separable((P,), penalties,
                           groups)
    initial = np.random.standard_normal(P)

    composite_form = rr.separable_problem.fromatom(penalty, loss)

    solver = rr.FISTA(composite_form)
    solver.debug = debug
    solver.fit(tol=1.0e-12, min_its=200)
    coefs = solver.composite.coefs

    # Solve the problem with X, which has been normalized in place
    loss2 = rr.quadratic_loss.affine(X, -Y, coef=coef)

    initial2 = np.random.standard_normal(P)
    composite_form2 = rr.separable_problem.fromatom(penalty, loss2)

    solver2 = rr.FISTA(composite_form2)
    solver2.debug = debug
    solver2.fit(tol=1.0e-12, min_its=200)
    coefs2 = solver2.composite.coefs

    for _ in range(10):
        beta = np.random.standard_normal(P)
        g1 = loss.smooth_objective(beta, mode='grad')
        g2 = loss2.smooth_objective(beta, mode='grad')
        np.testing.assert_almost_equal(g1, g2)
        b1 = penalty.proximal(sq(1, beta, g1,0))
        b2 = penalty.proximal(sq(1, beta, g2,0))
        np.testing.assert_almost_equal(b1, b2)

        f1 = composite_form.objective(beta)
        f2 = composite_form2.objective(beta)
        np.testing.assert_almost_equal(f1, f2)


    np.testing.assert_almost_equal(composite_form.objective(coefs), composite_form.objective(coefs2))
    np.testing.assert_almost_equal(composite_form2.objective(coefs), composite_form2.objective(coefs2))

    nt.assert_true(np.linalg.norm(coefs - coefs2) / max(np.linalg.norm(coefs),1) < 1.0e-04)

@set_seed_for_test()
def test_normalize_intercept():

    for issparse, value, inplace, intercept_column, scale, center in product([False, True], 
                                                       [1,3], 
                                                       [False, True], 
                                                       [None, 2],
                                                       [True, False],
                                                       [True, False]):
        
        print (issparse, value, inplace, intercept_column, scale, center)
        if not (issparse and inplace):

            X = np.random.standard_normal((20,6))
            if intercept_column is not None:
                X[:,intercept_column] = 1
            Y = X.copy()

            if issparse:
                X = scipy.sparse.csr_matrix(X)

            Xn = rr.normalize(X, 
                              value=value, 
                              inplace=inplace, 
                              intercept_column=intercept_column,
                              scale=scale, 
                              center=center)

            if intercept_column is not None:
                v = np.zeros(Y.shape[1])
                v[intercept_column] = 4
                yield np.testing.assert_allclose, Xn.linear_map(v), 4 * np.ones(Y.shape[0])

            if scale and center:

                Y -= Y.mean(0)[None,:]
                Y /= Y.std(0)[None,:]
                Y *= np.sqrt(value)
                if intercept_column is not None:
                    Y[:,intercept_column] = 1
            
            elif scale and not center:

                Y /= (np.sqrt((Y**2).sum(0))[None,:] / np.sqrt(Y.shape[0]))
                Y *= np.sqrt(value)
                if intercept_column is not None:
                    Y[:,intercept_column] = 1

            elif center and not scale:

                Y -= Y.mean(0)[None,:]
                if intercept_column is not None:
                    Y[:,intercept_column] = 1

            V = np.random.standard_normal((20, 3))
            U = np.random.standard_normal((6,4))

            Xn.adjoint_map(V)
            yield np.testing.assert_allclose, np.dot(Y, U), Xn.linear_map(np.array(U))
            yield np.testing.assert_allclose, np.dot(Y, U), Xn.affine_map(np.array(U))
            yield np.testing.assert_allclose, np.dot(Y, U[:,0]), Xn.linear_map(np.array(U[:,0]))
            yield np.testing.assert_allclose, np.dot(Y, U[:,0]), Xn.affine_map(np.array(U[:,0]))
            yield np.testing.assert_allclose, np.dot(Y.T, V), Xn.adjoint_map(V)
            yield nt.assert_raises, ValueError, Xn.linear_map, np.zeros((6,4,3))

            X2 = Xn.slice_columns(list(range(3)))
            Y2 = Y[:,:3]
            U2 = np.random.standard_normal((3,4))
            V2 = np.random.standard_normal(20)

            yield np.testing.assert_allclose, np.dot(Y2, U2), X2.linear_map(np.array(U2))
            yield np.testing.assert_allclose, np.dot(Y2, U2), X2.affine_map(np.array(U2))
            yield np.testing.assert_allclose, np.dot(Y2, U2[:,0]), X2.linear_map(np.array(U2[:,0]))
            yield np.testing.assert_allclose, np.dot(Y2, U2[:,0]), X2.affine_map(np.array(U2[:,0]))
            yield np.testing.assert_allclose, np.dot(Y2.T, V2), X2.adjoint_map(V2)

            X2 = Xn.slice_columns(list(range(3,6)))
            Y2 = Y[:,3:]
            U2 = np.random.standard_normal((3,4))
            V2 = np.random.standard_normal(20)

            yield np.testing.assert_allclose, np.dot(Y2, U2), X2.linear_map(np.array(U2))
            yield np.testing.assert_allclose, np.dot(Y2, U2), X2.affine_map(np.array(U2))
            yield np.testing.assert_allclose, np.dot(Y2, U2[:,0]), X2.linear_map(np.array(U2[:,0]))
            yield np.testing.assert_allclose, np.dot(Y2, U2[:,0]), X2.affine_map(np.array(U2[:,0]))
            yield np.testing.assert_allclose, np.dot(Y2.T, V2), X2.adjoint_map(V2)

            keep = np.zeros(6, np.bool)
            keep[:3] = 1
            X2 = Xn.slice_columns(keep)
            Y2 = Y[:,:3]
            U2 = np.random.standard_normal((3,4))
            V2 = np.random.standard_normal(20)

            yield np.testing.assert_allclose, np.dot(Y2, U2), X2.linear_map(np.array(U2))
            yield np.testing.assert_allclose, np.dot(Y2, U2), X2.affine_map(np.array(U2))
            yield np.testing.assert_allclose, np.dot(Y2, U2[:,0]), X2.linear_map(np.array(U2[:,0]))
            yield np.testing.assert_allclose, np.dot(Y2, U2[:,0]), X2.affine_map(np.array(U2[:,0]))
            yield np.testing.assert_allclose, np.dot(Y2.T, V2), X2.adjoint_map(V2)

    yield nt.assert_raises, ValueError, rr.normalize, scipy.sparse.csr_matrix(Y), True, True, 1, True




""" Implement doc templates using objective templates and vars
"""

from .doctemplates import doc_templater


def objective_doc_templater(doc_dict=None, doc_errors=True):
    """ Return doc_templater class generator for klasses with objective docs

    Check for specific case where class being decorated has attributes
    ``objective_template`` and ``objective_vars``.  Set new entry ``objective`` into class
    ``doc_dict`` using these class attributes.  Then continue decorating as for
    ``doc_templater``.
    """
    if doc_dict is None:
        doc_dict = {}
    def obj_kdec(klass):
        if (hasattr(klass, 'objective_template') and
            hasattr(klass, 'objective_vars')):
            doc_dict['objective'] = (klass.objective_template %
                                     klass.objective_vars)
            for k, v in klass.objective_vars.items():
                doc_dict[k] = v
        return doc_templater(doc_dict, doc_errors)(klass)
    return obj_kdec




""" Decorators for making and using docstring templates on methods

The doctest code uses class decorators and therefore needs at least Python 2.6.

If you have Python 2.5 you can decorate the class without using class
decorators::

    class C(object):
        @doc_template_provider
        def f(self):
            "A docstring %(var)s"
    C = doc_templater()(C)


Examples
--------

We can set a docstring template from a method docstring

>>> from regreg.doctemplates import *
>>> @doc_templater()
... class C(object):
...     @doc_template_provider
...     def f(self):
...         "A docstring %(var)s"

This wipes out the docstring and stores it for future use:

>>> C().f.__doc__ is None
True

We can use it with the ``doc_template_user`` decorator on the method

>>> @doc_templater(dict(var='something'))
... class D(C):
...     @doc_template_user
...     def f(self):
...         pass

>>> D().f.__doc__
'A docstring something'

The ``doc_dict`` parameter to the class decorator sets parameters for the
docstring for the class.  These override parameters from inherited classes:

>>> @doc_templater(dict(var='something else'))
... class E(D):
...     @doc_template_user
...     def f(self):
...         pass

>>> E().f.__doc__
'A docstring something else'

>>> @doc_templater(dict(var='something else again'))
... class F(E):
...     @doc_template_user
...     def f(self):
...         pass

>>> F().f.__doc__
'A docstring something else again'

The same method docstring can be a template and template user:

>>> @doc_templater(dict(foo='something else entirely'))
... class G(object):
...     @doc_template_user
...     @doc_template_provider
...     def f2(self):
...         "Look at %(foo)s"

>>> G().f2.__doc__
'Look at something else entirely'
"""

def doc_template_user(m):
    """ Decorator to mark method `m` as using docstring template

    If ``klass`` is the enclosing class of `m`, and ``klass`` is decorated with
    ``doc_templater``, then the docstring for `m` will be givem by
    ``klass._doc_templates[m.__name__] % klass._doc_dict``
    """
    m._uses_doc_template = True
    return m


def doc_template_provider(m):
    """ Decorator to mark docstring from method `m` as being a doc template

    We label the docstring of `m` (``m.__doc__``) to be put into the
    ``_doc_templates`` attribute of the enclosing class, when the klass is
    decorated with the ``doc_templater`` decorator.
    """
    m._doc_template = m.__doc__
    m.__doc__ = None
    return m


def doc_templater(doc_dict=None, doc_error=True):
    """ Return class decorator to record and provide doc templates

    Parameters
    ----------
    doc_dict : dict
        Extra key / value pairs for this class to fill template docstrings.
        Parameters in this input dict override parameters from inherited
        classes.
    doc_error : {True, False}, optional
        Whether to raise error in case of non-empty docstring that will be
        thrown away by using docstring template, where the docstring has not
        itself been marked for use as a template.

    Returns
    -------
    kdec : function
        Class decorator
    """
    if doc_dict is None:
        doc_dict = {}
    def kdec(klass):
        if hasattr(klass, '_doc_templates'): # inherited
            klass._doc_templates = klass._doc_templates.copy()
        else:
            klass._doc_templates = {}
        if hasattr(klass, '_doc_dict'): # inherited
            klass_doc_dict = klass._doc_dict.copy()
            klass_doc_dict.update(doc_dict)
        else:
            klass_doc_dict = doc_dict
        for obj in klass.__dict__.values():
            if hasattr(obj, '_doc_template') and hasattr(obj, '__name__'):
                klass._doc_templates[obj.__name__] = obj._doc_template
            if hasattr(obj, '_uses_doc_template') and hasattr(obj, '__name__'):
                if doc_error and not obj.__doc__ is None:
                    raise ValueError("Refusing to discard unexpected docstring for %s" % repr(obj) +
                                     " - set `doc_error` to False if you want " + 
                                     "to allow this")
                template = klass._doc_templates[obj.__name__]
                obj.__doc__ = template % klass_doc_dict
        klass._doc_dict = klass_doc_dict
        return klass
    return kdec



"""
https://gist.github.com/fabianp/3081831
"""
import numpy as np

# Author : Alexandre Gramfort
# license : BSD

def pav(y):
    """
    PAV uses the pair adjacent violators method to produce a monotonic
    smoothing of y

    translated from matlab by Sean Collins (2006) as part of the EMAP toolbox
    """
    y = np.asarray(y)
    assert y.ndim == 1
    n_samples = len(y)
    v = y.copy()
    lvls = np.arange(n_samples)
    lvlsets = np.c_[lvls, lvls]
    flag = 1
    while flag:
        deriv = np.diff(v)
        if np.all(deriv >= 0):
            break

        viol = np.where(deriv < 0)[0]
        start = lvlsets[viol[0], 0]
        last = lvlsets[viol[0] + 1, 1]
        s = 0
        n = last - start + 1
        for i in range(start, last + 1):
            s += v[i]

        val = s / n
        for i in range(start, last + 1):
            v[i] = val
            lvlsets[i, 0] = start
            lvlsets[i, 1] = last
    return v



from __future__ import print_function, division, absolute_import

from copy import copy
import nose.tools as nt

import numpy as np

import regreg.api as rr
from regreg.tests.decorators import set_seed_for_test

@set_seed_for_test()
def test_class():

    n, p = (10, 5)
    D = np.random.standard_normal((n,p))
    v = np.random.standard_normal(n)
    pen = rr.l1norm.affine(D, v, lagrange=0.4)

    pen2 = rr.l1norm(n, lagrange=0.4, offset=np.random.standard_normal(n))
    pen2.quadratic = None
    cls = type(pen)
    pen_aff = cls(pen2, rr.affine_transform(D, v))

    for _pen in [pen, pen_aff]:
        # Run to ensure code gets executed in tests (smoke test)
        print(_pen.dual)
        print(_pen.latexify())
        print(str(_pen))
        print(repr(_pen))
        print(_pen._repr_latex_())
        _pen.nonsmooth_objective(np.random.standard_normal(p))
        q = rr.identity_quadratic(0.5,0,0,0)
        smoothed_pen = _pen.smoothed(q)



import itertools
import numpy as np
import nose.tools as nt

import regreg.api as rr
from regreg.atoms.slope import slope, slope_conjugate, have_sklearn_iso
from regreg.tests.decorators import set_seed_for_test

from .test_seminorms import Solver, SolverFactory, all_close

def test_kkt():

    Z = np.array([0.75529996,
                  0.87948569,
                  -0.2399563,
                  -0.69505665,
                  -0.47140008,
                  0.34947641,
                  1.61145729,
                  0.23933927,
                  0.48500263,
                  -1.65860471,
                  2.67991031,
                  -1.10352213,
                  1.81778261,
                  -0.55604734,
                  0.95771124,
                  0.81522068,
                  0.0643194,
                  -0.16640537,
                  -0.94070977,
                  -0.0535945]) 

    o = np.argsort(np.fabs(Z))
    Z = Z[o][::-1]
    pen1 = slope(0.14 * np.arange(20)[::-1] / 20 + .14, lagrange=1.)
    pen1.check_subgradient(pen1, Z)

def test_l1norm_equivalent():
    """
    with equal weights the prox is the same as l1 norm
    """
    pen1 = slope(np.ones(3) * 4, lagrange=0.3)
    pen2 = rr.l1norm(3, lagrange=1.2)

    Z = np.array([3,2,4])
    np.testing.assert_allclose(pen1.lagrange_prox(Z), pen2.lagrange_prox(Z))

    Z = np.random.standard_normal(3) * 100
    np.testing.assert_allclose(pen1.lagrange_prox(Z), pen2.lagrange_prox(Z))

    dual1 = pen1.conjugate
    dual2 = pen2.conjugate

    np.testing.assert_allclose(Z, pen1.lagrange_prox(Z) + dual1.bound_prox(Z))
    np.testing.assert_allclose(dual1.bound_prox(Z), dual2.bound_prox(Z))

def test_duality():
    """
    with equal weights the prox is the same as l1 norm
    """
    pen1 = slope(np.array([0.4,0.35,0.1]), lagrange=1.1)
    dual1 = pen1.conjugate
    Z = np.random.standard_normal(3) * 100
    np.testing.assert_allclose(Z, pen1.lagrange_prox(Z) + dual1.bound_prox(Z))

class SlopeSolverFactory(SolverFactory):

    weight_choices = [np.arange(20)[::-1]/20. + 1]
    FISTA_choices = [True]
    L_choices = [0.3]
    coef_stop_choices = [False]

    def __init__(self, klass, mode, use_sklearn=True):
        self.klass = klass
        self.mode = mode
        self.use_sklearn = use_sklearn

    def __iter__(self):
        for offset, FISTA, coef_stop, L, q, w in itertools.product(self.offset_choices,
                                                                   self.FISTA_choices,
                                                                   self.coef_stop_choices,
                                                                   self.L_choices,
                                                                   self.quadratic_choices,
                                                                   self.weight_choices):
            self.FISTA = FISTA
            self.coef_stop = coef_stop
            self.L = L

            if self.mode == 'lagrange':
                atom = self.klass(w, lagrange=self.lagrange)
            else:
                atom = self.klass(w, bound=self.bound)
            atom.use_sklearn = self.use_sklearn and have_sklearn_iso # test out both prox maps if available

            if q: 
                atom.quadratic = rr.identity_quadratic(0, 0, np.random.standard_normal(atom.shape)*0.02)

            if offset:
                atom.offset = 0.02 * np.random.standard_normal(atom.shape)

            solver = Solver(atom, interactive=self.interactive, 
                            coef_stop=coef_stop,
                            FISTA=FISTA,
                            L=L)
            yield solver

@set_seed_for_test()
@np.testing.dec.slow
def test_proximal_maps():
    for klass, mode in zip([slope, slope_conjugate], ['lagrange', 'bound']):
        factory = SlopeSolverFactory(klass, mode)
        for solver in factory:
            for t in solver.all_tests():
                yield t
        factory = SlopeSolverFactory(klass, mode, use_sklearn=False)
        for solver in factory:
            for t in solver.all_tests():
                yield t





from __future__ import print_function, division, absolute_import

import itertools
import nose.tools as nt

import numpy as np

import regreg.atoms.group_lasso as GL
import regreg.api as rr

from .test_seminorms import Solver, all_close, SolverFactory
from .test_cones import ConeSolverFactory
from regreg.tests.decorators import set_seed_for_test

class GroupSolverFactory(SolverFactory):

    group_choices = [np.arange(10),
                     np.array([1,1,2,2,2,3,3,4,4,4,4,5,5,6,6,6,6])]
    weight_choices = [{}, {}]

    FISTA_choices = [True]
    L_choices = [0.3]
    coef_stop_choices = [False]

    def __init__(self, klass, mode):
        self.klass = klass
        self.mode = mode

    def __iter__(self):
        for offset, FISTA, coef_stop, L, q, groups, w in itertools.product(self.offset_choices,
                                                                           self.FISTA_choices,
                                                                           self.coef_stop_choices,
                                                                           self.L_choices,
                                                                           self.quadratic_choices,
                                                                           self.group_choices,
                                                                           self.weight_choices):
            self.FISTA = FISTA
            self.coef_stop = coef_stop
            self.L = L

            if self.mode == 'lagrange':
                atom = self.klass(groups, w, lagrange=self.lagrange)
            else:
                atom = self.klass(groups, w, bound=self.bound)

            if q: 
                atom.quadratic = rr.identity_quadratic(0,0,np.random.standard_normal(atom.shape)*0.02)

            if offset:
                atom.offset = 0.02 * np.random.standard_normal(atom.shape)

            solver = Solver(atom, interactive=self.interactive, 
                            coef_stop=coef_stop,
                            FISTA=FISTA,
                            L=L)
            yield solver


class GroupConeSolverFactory(ConeSolverFactory):

    group_choices = [np.arange(10),
                     np.array([1,1,2,2,2,3,3,4,4,4,4,5,5,6,6,6,6])]
    weight_choices = [{}, {}]

    def __iter__(self):
        for offset, FISTA, coef_stop, L, q, groups, w in itertools.product(self.offset_choices,
                                                                           self.FISTA_choices,
                                                                           self.coef_stop_choices,
                                                                           self.L_choices,
                                                                           self.quadratic_choices,
                                                                           self.group_choices,
                                                                           self.weight_choices):
            self.FISTA = FISTA
            self.coef_stop = coef_stop
            self.L = L

            atom = self.klass(groups, w)

            if q: 
                atom.quadratic = rr.identity_quadratic(0,0,np.random.standard_normal(atom.shape)*0.02)

            if offset:
                atom.offset = 0.02 * np.random.standard_normal(atom.shape)

            solver = Solver(atom, interactive=self.interactive, 
                            coef_stop=coef_stop,
                            FISTA=FISTA,
                            L=L)
            yield solver


@set_seed_for_test()
@np.testing.dec.slow
def test_proximal_maps(interactive=False):
    for klass in GL.conjugate_seminorm_pairs.keys(): 
        factory = GroupSolverFactory(klass, 'lagrange')
        for solver in factory:
            penalty = solver.atom
            dual = penalty.conjugate 
            Z = solver.prox_center
            L = solver.L

            yield all_close, penalty.lagrange_prox(Z, lipschitz=L), Z-dual.bound_prox(Z*L)/L, 'testing lagrange_prox and bound_prox starting from atom\n %s ' % klass, None

            # some arguments of the constructor

            nt.assert_raises(AttributeError, setattr, penalty, 'bound', 4.)
            nt.assert_raises(AttributeError, setattr, dual, 'lagrange', 4.)
        
            nt.assert_raises(AttributeError, setattr, penalty, 'bound', 4.)
            nt.assert_raises(AttributeError, setattr, dual, 'lagrange', 4.)

            for t in solver.all_tests():
                yield t

        factory = GroupSolverFactory(klass, 'bound')
        for solver in factory:
            for t in solver.all_tests():
                yield t

    for klass in GL.conjugate_cone_pairs.keys():
        factory = GroupConeSolverFactory(klass)
        for solver in factory:
            for t in solver.all_tests():
                yield t




import numpy as np
from scipy import sparse
from copy import copy
import warnings

from .seminorms import seminorm as unweighted_seminorm

from ..problems.composite import composite, nonsmooth, smooth_conjugate
from ..affine import (linear_transform, identity as identity_transform, 
                     affine_transform, selector)
from ..identity_quadratic import identity_quadratic
from ..atoms import _work_out_conjugate
from ..objdoctemplates import objective_doc_templater
from ..doctemplates import (doc_template_user, doc_template_provider)
from .piecewise_linear import find_solution_piecewise_linear

class seminorm(unweighted_seminorm):

    """
    A class that defines the API for support functions.
    """
    tol = 1.0e-05

    def __init__(self, weights, lagrange=None, bound=None, 
                 offset=None, 
                 quadratic=None,
                 initial=None):

        self.weights = np.asarray(weights, np.float)
        unweighted_seminorm.__init__(self, self.weights.shape,
                                     lagrange=lagrange,
                                     bound=bound,
                                     quadratic=quadratic,
                                     initial=initial,
                                     offset=offset)

        self.invweights = np.zeros_like(self.weights)
        zero_weight = self.weights == 0
        self.invweights[~zero_weight] = 1. / self.weights[~zero_weight]
        self.invweights[zero_weight] = np.inf
        
        if not np.all(self.weights >= 0):
            raise ValueError('weights should be nonnegative')

    def __eq__(self, other):
        if self.__class__ == other.__class__:
            weights_equal = np.all(np.equal(self.weights, other.weights))
            if self.bound is not None:
                return self.bound == other.bound and weights_equal
            return self.lagrange == other.lagrange and weights_equal
        return False

    def __copy__(self):
        return self.__class__(self.weights.copy(),
                              quadratic=self.quadratic,
                              initial=self.coefs,
                              bound=copy(self.bound),
                              lagrange=copy(self.lagrange),
                              offset=copy(self.offset))
    
    def __repr__(self):
        if self.lagrange is not None:
            if not self.quadratic.iszero:
                return "%s(%s, lagrange=%f, offset=%s)" % \
                    (self.__class__.__name__,
                     str(self.weights),
                     self.lagrange,
                     str(self.offset))
            else:
                return "%s(%s, lagrange=%f, offset=%s, quadratic=%s)" % \
                    (self.__class__.__name__, 
                     str(self.weights),
                     self.lagrange,
                     str(self.offset),
                     self.quadratic)
        else:
            if not self.quadratic.iszero:
                return "%s(%s, bound=%f, offset=%s)" % \
                    (self.__class__.__name__,
                     str(self.weights),
                     self.bound,
                     str(self.offset))
            else:
                return "%s(%s, bound=%f, offset=%s, quadratic=%s)" % \
                    (self.__class__.__name__,
                     str(self.weights),
                     self.bound,
                     str(self.offset),
                     self.quadratic)

    @doc_template_user
    def get_conjugate(self):
        if self.quadratic.coef == 0:

            offset, outq = _work_out_conjugate(self.offset, self.quadratic)

            if self.bound is None:
                cls = conjugate_weighted_pairs[self.__class__]
                atom = cls(self.invweights, 
                           bound=self.lagrange, 
                           lagrange=None,
                           offset=offset,
                           quadratic=outq)
            else:
                cls = conjugate_weighted_pairs[self.__class__]
                atom = cls(self.invweights,
                           lagrange=self.bound, 
                           bound=None,
                           offset=offset,
                           quadratic=outq)
        else:
            atom = smooth_conjugate(self)

        self._conjugate = atom
        self._conjugate._conjugate = self
        return self._conjugate
    conjugate = property(get_conjugate)
    
    def form_transform(self, subsample=False):
        '''
        By subsampling we can get rid of some variables that have 0 weights.
        '''
        if not hasattr(self, '_linear_transform'):
            if self.weights is not None:
                test = self.weights == 0
                if test.sum() and subsample:
                    self._linear_transform = selector(~test, self.shape)
                else:
                    self._linear_transform = identity_transform(self.shape)
            else:
                self._linear_transform = identity_transform(self.shape)
        return self._linear_transform
    linear_transform = property(form_transform)


@objective_doc_templater()
class l1norm(seminorm):

    """
    The l1 norm
    """
    prox_tol = 1.0e-10

    objective_template = r"""\|W%(var)s\|_1"""

    @doc_template_user
    def seminorm(self,
                 arg,
                 lagrange=None,
                 check_feasibility=False):
        lagrange = seminorm.seminorm(self,
                                     arg, 
                                     check_feasibility=check_feasibility, 
                                     lagrange=lagrange)
        finite = np.isfinite(self.weights)
        if check_feasibility:
            check_zero = ~finite * (arg != 0)
            if check_zero.sum():
                return np.inf
        arg = np.asarray(arg)
        return lagrange * np.fabs(arg[finite] * self.weights[finite]).sum()

    @doc_template_user
    def constraint(self, arg, bound=None):
        bound = seminorm.constraint(self, arg, bound=bound)
        inbox = self.seminorm(arg,
                              lagrange=1,
                              check_feasibility=True) <= bound * (1 + self.tol)
        if inbox:
            return 0
        else:
            return np.inf

    @doc_template_user
    def lagrange_prox(self, arg,  lipschitz=1, lagrange=None):
        lagrange = seminorm.lagrange_prox(self, arg, lipschitz, lagrange)
        return np.sign(arg) * np.maximum(np.fabs(arg)-lagrange * self.weights
                                       / lipschitz, 0)

    @doc_template_user
    def bound_prox(self, arg, bound=None):
        bound = seminorm.constraint(self, arg, bound=bound)
        _isinf = np.isinf(self.weights)
        _is0 = self.weights == 0
        _keep = ~(_isinf + _is0)
        _lagrange = find_solution_piecewise_linear(bound,
                                                   0,
                                                   np.fabs(arg[_keep]),
                                                   self.weights[_keep])
        cut = self.weights * (_lagrange + _isinf)
        value = np.sign(arg) * np.maximum(np.fabs(arg) - cut, 0)
        return value
    
    def terms(self, arg):
        """
        Return the args that are summed
        in computing the seminorm.

        """
        arg = np.asarray(arg)
        return np.fabs(arg) * self.weights

@objective_doc_templater()
class supnorm(seminorm):

    r"""
    The :math:`\ell_{\infty}` norm
    """

    objective_template = r"""\|W%(var)s\|_{\infty}"""

    @doc_template_user
    def seminorm(self,
                 arg,
                 lagrange=None,
                 check_feasibility=False):
        lagrange = seminorm.seminorm(self,
                                     arg, 
                                     check_feasibility=check_feasibility, 
                                     lagrange=lagrange)
        finite = np.isfinite(self.weights)
        if check_feasibility:
            check_zero = ~finite * (arg != 0)
            if check_zero.sum():
                return np.inf
        return lagrange * np.fabs(arg[finite] * self.weights[finite]).max()

    @doc_template_user
    def constraint(self, arg, bound=None):
        bound = seminorm.constraint(self, arg, bound=bound)
        inbox = self.seminorm(arg, lagrange=1,
                              check_feasibility=True) <= bound * (1+self.tol)
        if inbox:
            return 0
        else:
            return np.inf

    @doc_template_user
    def lagrange_prox(self, arg,  lipschitz=1, lagrange=None):
        lagrange = seminorm.lagrange_prox(self, arg, lipschitz, lagrange)
        weights = self.invweights
        _isinf = np.isinf(weights)
        _is0 = weights == 0
        _keep = ~(_isinf + _is0)
        _lagrange = find_solution_piecewise_linear(lagrange / lipschitz,
                                                   0,
                                                   np.fabs(arg[_keep]),
                                                   weights[_keep])
        resid = np.sign(arg) * np.maximum(np.fabs(arg)-_lagrange * weights, 0)

        return arg - resid

    @doc_template_user
    def bound_prox(self, arg, bound=None):
        bound = seminorm.bound_prox(self, arg, bound)
        return np.clip(arg, -bound * self.invweights, bound * self.invweights)

    def terms(self, arg, check_feasibility=True):
        """
        Return the args that are maximized
        in computing the seminorm.

        """
        arg = np.asarray(arg)
        v = np.fabs(arg) / self.weights
        if check_feasibility:
            v[self.weights == 0] = np.where(np.fabs(arg[self.weights == 0]) != 0, np.inf, 0)
        return v

conjugate_weighted_pairs = {}
for n1, n2 in [(l1norm,supnorm)]:
    conjugate_weighted_pairs[n1] = n2
    conjugate_weighted_pairs[n2] = n1



from copy import copy
import warnings

from scipy import sparse
import numpy as np

from ..problems.composite import nonsmooth, smooth_conjugate
from ..affine import linear_transform, identity as identity_transform
from ..identity_quadratic import identity_quadratic
from ..smooth import affine_smooth
from ..atoms import _work_out_conjugate, atom, affine_atom
from ..objdoctemplates import objective_doc_templater
from ..doctemplates import (doc_template_user, doc_template_provider)

from .projl1_cython import projl1_epigraph

@objective_doc_templater()
class cone(atom):

    """
    A class that defines the API for cone constraints.
    """

    objective_template = r'\|%(var)s\|'
    objective_vars = {'var': r'\beta', 
                      'shape':'p', 
                      'linear':'D', 
                      'offset':r'\alpha',
                      'coneklass':'nonnegative',
                      'dualconeklass':'nonpositive',
                      'initargs':'(30,)', # args need to construct penalty
                      }
    tol = 1.0e-05

    def __eq__(self, other):
        if self.__class__ == other.__class__:
            return self.shape == other.shape
        return False

    def __copy__(self):
        return self.__class__(copy(self.shape),
                              offset=copy(self.offset),
                              initial=copy(self.coefs),
                              quadratic=copy(self.quadratic))
    
    def __repr__(self):
        if self.quadratic.iszero:
            return "%s(%s, offset=%s)" % \
                (self.__class__.__name__,
                 repr(self.shape), 
                 repr(self.offset))
        else:
            return "%s(%s, offset=%s, quadratic=%s)" % \
                (self.__class__.__name__,
                 repr(self.shape), 
                 repr(self.offset),
                 repr(self.quadratic))

    @doc_template_user
    @doc_template_provider
    def get_conjugate(self):
        """
        Return the conjugate of an given atom.

        >>> import regreg.api as rr
        >>> penalty = rr.%(coneklass)s(%(initargs)s)
        >>> penalty.get_conjugate() # doctest: +SKIP
        %(dualconeklass)s(%(initargs)s, offset=None)

        """
        if self.quadratic.coef == 0:
            offset, outq = _work_out_conjugate(self.offset, 
                                               self.quadratic)
            cls = conjugate_cone_pairs[self.__class__]
            atom = cls(self.shape, 
                       offset=offset,
                       quadratic=outq)
        else:
            atom = smooth_conjugate(self)
        self._conjugate = atom
        self._conjugate._conjugate = self
        return self._conjugate
    conjugate = property(get_conjugate)

    @doc_template_user
    @doc_template_provider
    def get_dual(self):
        r"""
        Return the dual of an atom. This dual is formed by making the  
        substitution $v=Ax$ where $A$ is the `self.linear_transform`.

        >>> import regreg.api as rr
        >>> penalty = rr.%(coneklass)s(%(initargs)s)
        >>> penalty # doctest: +SKIP
        %(coneklass)s(%(initargs)s, offset=None)
        >>> penalty.dual # doctest: +SKIP
        (<regreg.affine.identity object at 0x...>, %(dualconeklass)s(%(initargs)s, offset=None))

        If there is a linear part to the penalty, the linear_transform may not be identity:

        >>> D = (np.identity(4) + np.diag(-np.ones(3),1))[:-1]
        >>> D
        array([[ 1., -1.,  0.,  0.],
               [ 0.,  1., -1.,  0.],
               [ 0.,  0.,  1., -1.]])
        >>> linear_atom = rr.nonnegative.linear(D)
        >>> linear_atom # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
        affine_cone(nonnegative((3,), offset=None), array([[ 1., -1.,  0.,  0.],
               [ 0.,  1., -1.,  0.],
               [ 0.,  0.,  1., -1.]]))
        >>> linear_atom.dual # doctest: +ELLIPSIS
        (<regreg.affine.linear_transform object at 0x...>, nonpositive((3,), offset=None))

        """
        return self.linear_transform, self.conjugate

    @property
    def linear_transform(self):
        if not hasattr(self, "_linear_transform"):
            self._linear_transform = identity_transform(self.shape)
        return self._linear_transform
    
    @doc_template_user
    @doc_template_provider
    def constraint(self, x):
        """
        The constraint

        .. math::

           %(objective)s
        """
        raise NotImplementedError

    @doc_template_user
    @doc_template_provider
    def nonsmooth_objective(self, x, check_feasibility=False):
        '''
        >>> import regreg.api as rr
        >>> cone = rr.nonnegative(4)
        >>> cone.nonsmooth_objective([3, 4, 5, 9])
        0.0
        '''
        arg = np.asarray(x)
        x_offset = self.apply_offset(x)
        if check_feasibility:
            v = self.constraint(x_offset)
        else:
            v = 0
        v += self.quadratic.objective(arg, 'func')
        return v

    @doc_template_user
    @doc_template_provider
    def proximal(self, quadratic, prox_control=None):
        r"""
        The proximal operator. 

        .. math::

           v^{\lambda}(x) = \text{argmin}_{v \in \mathbb{R}^{%(shape)s}} \frac{L}{2}
           \|x-\alpha - v\|^2_2 + %(objective)s + \langle v, \eta \rangle

        where :math:`\alpha` is `self.offset`,
        :math:`\eta` is `quadratic.linear_term`.

        >>> import regreg.api as rr
        >>> cone = rr.nonnegative((4,))
        >>> Q = rr.identity_quadratic(1.5, [3, -4, -1, 1], 0, 0)
        >>> np.allclose(cone.proximal(Q), [3, 0, 0, 1]) # doctest: +NORMALIZE_WHITESPACE
        True

        Parameters
        ----------

        quadratic : `regreg.identity_quadratic.identity_quadratic`

            A quadratic added to the atom before minimizing.

        prox_control : `[None, dict]`

            This argument is ignored for seminorms, but otherwise
            is passed to `regreg.algorithms.FISTA` if the atom
            needs to be solved iteratively.

        Returns
        -------

        Z : `np.ndarray(np.float)`
            The proximal map of the implied center of `quadratic`.

        """
        offset, totalq = (self.quadratic + quadratic).recenter(self.offset)
        if totalq.coef == 0:
            raise ValueError('lipschitz + quadratic coef must be positive')

        prox_arg = -totalq.linear_term / totalq.coef

        eta = self.cone_prox(prox_arg)
        if offset is None:
            return eta
        else:
            return eta + offset

    @doc_template_user
    @doc_template_provider
    def cone_prox(self, x):
        r"""
        Return (unique) minimizer

        .. math::

           %(var)s^{\lambda}(u) = \text{argmin}_{%(var)s \in \mathbb{R}^%(shape)s} 
           \frac{1}{2} \|%(var)s-u\|^2_2 + %(objective)s

        """
        raise NotImplementedError

    # the minus signs below for offset is there until affine transforms SUBTRACT 
    # their offset until add. 
    # for atoms, the offset is really the "center"

    @classmethod
    @doc_template_provider
    def linear(cls, linear_operator, diag=False,
               offset=None,
               quadratic=None):
        """
        Composition of a cone constraint and a linear
        transform.
        """

        if not isinstance(linear_operator, linear_transform):
            l = linear_transform(linear_operator, diag=diag)
        else:
            l = linear_operator
        if offset is None:
            offset = 0
        cone = cls(l.output_shape, 
                   offset=-offset,
                   quadratic=quadratic)
        return affine_cone(cone, l)

    @classmethod
    @doc_template_provider
    def affine(cls, linear_operator, offset, diag=False,
               quadratic=None):
        """
        Composition of a cone constraint and a linear
        transform.
        """
        if not isinstance(linear_operator, linear_transform):
            l = linear_transform(linear_operator, diag=diag)
        else:
            l = linear_operator
        if offset is None:
            offset = 0
        cone = cls(l.output_shape, 
                   offset=-offset,
                   quadratic=quadratic)
        return affine_cone(cone, l)

    @staticmethod
    def check_subgradient(atom, prox_center):
        r"""
        For a given seminorm, verify the KKT condition for
        the problem for the proximal problem

        .. math::

           \text{minimize}_u \frac{1}{2} \|u-z\|^2_2 + h(z)

        where $z$ is the `prox_center` and $h$ is `atom`.

        This should return two values that are 0, 
        one is the inner product of the minimizer and the residual, the
        other is just 0.

        Parameters
        ----------

        atom : `cone`
             A cone instance with a `proximal` method.

        prox_center : np.ndarray(np.float)
             Center for the proximal map.

        Returns
        -------

        v1, v2 : float
             Two values that should be equal if the proximal map is correct.

        """
        atom = copy(atom)
        atom.quadratic = identity_quadratic(0,0,0,0)
        atom.offset = None
        q = identity_quadratic(1, prox_center, 0, 0)
        U = atom.proximal(q)
        return ((prox_center - U) * U).sum(), 0


@objective_doc_templater()
class affine_cone(affine_atom):

    def __repr__(self):
        return "affine_cone(%s, %s)" % (repr(self.atom),
                                        repr(self.linear_transform.linear_operator))

@objective_doc_templater()
class nonnegative(cone):
    """
    The non-negative cone constraint (which is the support
    function of the non-positive cone constraint).
    """

    objective_template = r"""I^{\infty}(%(var)s \succeq 0)"""
    objective_vars = cone.objective_vars.copy()
    objective_vars['coneklass'] = 'nonnegative'
    objective_vars['dualconeklass'] = 'nonpositive'

    @doc_template_user
    def constraint(self, x):
        tol_lim = np.fabs(x).max() * self.tol
        incone = np.all(np.greater_equal(x, -tol_lim))
        if incone:
            return 0
        return np.inf

    @doc_template_user
    def cone_prox(self, x):
        return np.maximum(x, 0)

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return cone.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_conjugate(self):
        return cone.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return cone.dual(self)

@objective_doc_templater()
class nonpositive(nonnegative):

    """
    The non-positive cone constraint (which is the support
    function of the non-negative cone constraint).
    """

    objective_template = r"""I^{\infty}(%(var)s \preceq 0)"""
    objective_vars = cone.objective_vars.copy()
    objective_vars['dualconeklass'] = 'nonnegative'
    objective_vars['coneklass'] = 'nonpositive'

    @doc_template_user
    def constraint(self, x):
        tol_lim = np.fabs(x).max() * self.tol
        incone = np.all(np.less_equal(x, tol_lim))
        if incone:
            return 0
        return np.inf

    @doc_template_user
    def cone_prox(self, x):
        return np.minimum(x, 0)

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return cone.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_conjugate(self):
        return cone.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return cone.dual(self)

@objective_doc_templater()
class zero(cone):
    """
    The zero seminorm, support function of :math:\{0\}
    """

    objective_template = r"""{\cal Z}(%(var)s)"""
    objective_vars = cone.objective_vars.copy()
    objective_vars['coneklass'] = 'zero'
    objective_vars['dualconeklass'] = 'zero_constraint'

    @doc_template_user
    def constraint(self, x):
        return 0.

    @doc_template_user
    def cone_prox(self, x):
        return np.asarray(x)

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return cone.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_conjugate(self):
        return cone.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return cone.dual(self)

@objective_doc_templater()
class zero_constraint(cone):
    """
    The zero constraint, support function of :math:`\mathbb{R}`^p
    """

    objective_template = r"""I^{\infty}(%(var)s = 0)"""
    objective_vars = cone.objective_vars.copy()
    objective_vars['coneklass'] = 'zero_constraint'
    objective_vars['dualconeklass'] = 'zero'

    @doc_template_user
    def constraint(self, x):
        if not np.linalg.norm(x) <= self.tol:
            return np.inf
        return 0.

    @doc_template_user
    def cone_prox(self, x):
        return np.zeros(np.asarray(x).shape)

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return cone.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_conjugate(self):
        return cone.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return cone.dual(self)

@objective_doc_templater()
class l2_epigraph(cone):

    """
    The l2_epigraph constraint.
    """

    objective_template = r"""I^{\infty}(\|%(var)s[:-1]\|_2 \leq %(var)s[-1])"""
    objective_vars = cone.objective_vars.copy()
    objective_vars['coneklass'] = 'l2_epigraph'
    objective_vars['dualconeklass'] = 'l2_epigraph_polar'

    @doc_template_user
    def constraint(self, x):
        
        incone = np.linalg.norm(x[:-1]) <= (1 + self.tol) * x[-1]
        if incone:
            return 0
        return np.inf

    @doc_template_user
    def cone_prox(self, x):
        norm = x[-1]
        coef = x[:-1]
        norm_coef = np.linalg.norm(coef)
        thold = (norm_coef - norm) / 2.
        result = np.zeros_like(x)
        result[:-1] = coef / norm_coef * max(norm_coef - thold, 0)
        result[-1] = max(norm + thold, 0)
        return result

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return cone.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_conjugate(self):
        return cone.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return cone.dual(self)

@objective_doc_templater()
class l2_epigraph_polar(cone):

    """
    The polar of the l2_epigraph constraint, which is the negative of the 
    l2 epigraph..
    """

    objective_template = r"""I^{\infty}(\|%(var)s[:-1]\|_2 \in -%(var)s[-1])"""
    objective_vars = cone.objective_vars.copy()
    objective_vars['dualconeklass'] = 'l2_epigraph'
    objective_vars['coneklass'] = 'l2_epigraph_polar'

    @doc_template_user
    def constraint(self, x):
        incone = np.linalg.norm(x[:-1]) <= (1 + self.tol) * (-x[-1])
        if incone:
            return 0
        return np.inf

    @doc_template_user
    def cone_prox(self, arg):
        arg = -arg
        norm = arg[-1]
        coef = arg[:-1]
        norm_coef = np.linalg.norm(coef)
        thold = (norm_coef - norm) / 2.
        result = np.zeros_like(arg)
        result[:-1] = coef / norm_coef * max(norm_coef - thold, 0)
        result[-1] = max(norm + thold, 0)
        return -result

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return cone.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_conjugate(self):
        return cone.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return cone.dual(self)

@objective_doc_templater()
class l1_epigraph(cone):

    """
    The l1_epigraph constraint.
    """

    objective_template = r"""I^{\infty}(\|%(var)s[:-1]\|_1 \leq %(var)s[-1])"""
    objective_vars = cone.objective_vars.copy()
    objective_vars['dualconeklass'] = 'l1_epigraph'
    objective_vars['coneklass'] = 'l1_epigraph_polar'

    @doc_template_user
    def constraint(self, x):
        incone = np.fabs(x[:-1]).sum() <= (1 + self.tol) * x[-1]
        if incone:
            return 0
        return np.inf

    @doc_template_user
    def cone_prox(self, x):
        return projl1_epigraph(x)

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return cone.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_conjugate(self):
        return cone.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return cone.dual(self)

@objective_doc_templater()
class l1_epigraph_polar(cone):

    """
    The polar l1_epigraph constraint.
    """

    objective_template = r"""I^{\infty}(\|%(var)s[:-1]\|_{\infty} \leq - %(var)s[-1])"""
    objective_vars = cone.objective_vars.copy()
    objective_vars['coneklass'] = 'l1_epigraph_polar'
    objective_vars['dualconeklass'] = 'l1_epigraph'

    @doc_template_user
    def constraint(self, x):
        
        incone = np.fabs(-x[:-1]).max() <= (1 + self.tol) * ( -x[-1])
        if incone:
            return 0
        return np.inf

    @doc_template_user
    def cone_prox(self, arg):
        arg = np.asarray(arg, np.float).copy()
        return arg - projl1_epigraph(arg)

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return cone.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_conjugate(self):
        return cone.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return cone.dual(self)

@objective_doc_templater()
class linf_epigraph(cone):

    """
    The $\ell_{\nfty}$ epigraph constraint.
    """

    objective_template = r"""I^{\infty}(\|%(var)s[:-1]\|_{\infty} \leq %(var)s[-1])"""
    objective_vars = cone.objective_vars.copy()
    objective_vars['coneklass'] = 'linf_epigraph'
    objective_vars['dualconeklass'] = 'linf_epigraph_polar'

    @doc_template_user
    def constraint(self, x):
        incone = np.fabs(x[:-1]).max() <= (1 + self.tol) * x[-1]
        if incone:
            return 0
        return np.inf

    @doc_template_user
    def cone_prox(self, arg):
        arg = np.asarray(arg, np.float)
        return arg + projl1_epigraph(-arg)

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return cone.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_conjugate(self):
        return cone.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return cone.dual(self)

@objective_doc_templater()
class linf_epigraph_polar(cone):

    """
    The polar linf_epigraph constraint which is just the
    negative of the l1_epigraph.
    """

    objective_template = r"""I^{\infty}(\|%(var)s[:-1]\|_1 \leq -%(var)s[-1])"""
    objective_vars = cone.objective_vars.copy()
    objective_vars['coneklass'] = 'linf_epigraph_polar'
    objective_vars['dualconeklass'] = 'linf_epigraph'

    @doc_template_user
    def constraint(self, x):
        incone = np.fabs(x[:-1]).sum() <= (1 + self.tol) * (-x[-1])
        if incone:
            return 0
        return np.inf

    @doc_template_user
    def cone_prox(self, arg):
        arg = np.asarray(arg, np.float)
        return -projl1_epigraph(-arg)

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return cone.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_conjugate(self):
        return cone.get_conjugate(self)

    @doc_template_user
    def get_dual(self):
        return cone.dual(self)

conjugate_cone_pairs = {}
for n1, n2 in [(nonnegative,nonpositive),
               (zero, zero_constraint),
               (l1_epigraph, l1_epigraph_polar),
               (l2_epigraph, l2_epigraph_polar),
               (linf_epigraph, linf_epigraph_polar)
               ]:
    conjugate_cone_pairs[n1] = n2
    conjugate_cone_pairs[n2] = n1




from copy import copy

import numpy as np
from scipy.stats import rankdata

from . import smooth_atom, affine_smooth
from .cox_utils import (cox_objective,
                        cox_gradient,
                        cox_hessian)

class cox_loglike(smooth_atom):

    """
    A class for combining the logistic log-likelihood with a general seminorm
    """

    objective_template = r"""\ell^{\text{Cox}}\left(%(var)s\right)"""

    def __init__(self, 
                 shape, 
                 event_times,
                 censoring,
                 coef=1., 
                 offset=None,
                 quadratic=None,
                 initial=None,
                 case_weights=None):

        smooth_atom.__init__(self,
                             shape,
                             offset=offset,
                             quadratic=quadratic,
                             initial=initial,
                             coef=coef)

        self.data = np.asarray([event_times, censoring]).T

        self._ordering = np.argsort(self.event_times).astype(np.intp)
        self._rankmax = (rankdata(self.event_times, method='max') - 1).astype(np.intp)
        self._rankmin = (rankdata(self.event_times, method='min') - 1).astype(np.intp)

        if case_weights is not None:
            case_weights = np.asarray(case_weights).astype(np.float)
            if not np.all(case_weights >= 0):
                raise ValueError('case_weights should be non-negative')
            self.case_weights = np.asarray(case_weights)
        else:
            self.case_weights = np.ones(self.event_times.shape, np.float)

        # buffers to store results used by C code
        self._G = np.zeros(self.event_times.shape, np.float) # gradient 
        self._exp_buffer = np.zeros(self.event_times.shape, np.float) # exp(eta)
        self._exp_accum = np.zeros(self.event_times.shape, np.float) # accum of exp(eta)
        self._expZ_accum = np.zeros(self.event_times.shape, np.float) # accum of Z*exp(eta)
        self._outer_1st = np.zeros(self.event_times.shape, np.float) # for log(W)
        self._outer_2nd = np.zeros(self.event_times.shape, np.float) # used in Hessian

    def smooth_objective(self, 
                         natural_param, 
                         mode='both', 
                         check_feasibility=False,
                         case_weights=None):
        """

        Evaluate the smooth objective, computing its value, gradient or both.

        Parameters
        ----------

        natural_param : ndarray
            The current parameter values.

        mode : str
            One of ['func', 'grad', 'both']. 

        check_feasibility : bool
            If True, return `np.inf` when
            point is not feasible, i.e. when `natural_param` is not
            in the domain.

        case_weights : ndarray
            Non-negative case weights

        Returns
        -------

        If `mode` is 'func' returns just the objective value 
        at `natural_param`, else if `mode` is 'grad' returns the gradient
        else returns both.
        """
        
        eta = natural_param # shorthand

        eta = self.apply_offset(eta)

        if case_weights is None:
            case_weights = np.ones_like(natural_param)
        cw = case_weights
        if self.case_weights is not None:
            cw *= self.case_weights

        censoring = self.censoring

        if mode in ['both', 'grad']:
            G = cox_gradient(self._G,
                             eta,
                             self._exp_buffer,
                             self._exp_accum,
                             self._outer_1st,
                             cw,
                             censoring,
                             self._ordering,
                             self._rankmin,
                             self._rankmax,
                             eta.shape[0])

        if mode in ['both', 'func']:
            F = cox_objective(eta,
                              self._exp_buffer,
                              self._exp_accum,
                              self._outer_1st,
                              cw,
                              censoring,
                              self._ordering,
                              self._rankmin,
                              self._rankmax,
                              eta.shape[0])

        if mode == 'both':
            return self.scale(F), self.scale(G)
        elif mode == 'grad':
            return self.scale(G)
        elif mode == 'func':
            return self.scale(F)
        else:
            raise ValueError("mode incorrectly specified")

    # Begin loss API

    def hessian_mult(self, 
                     natural_param, 
                     right_vector,
                     case_weights=None):
        """
        Evaluate Hessian of the loss at a pair of vectors.

        Parameters
        ----------

        natural_param : ndarray
            Parameters where Hessian will be evaluated.

        left_vec : ndarray
            Vector on the left in Hessian evaluation.

        right_vec : ndarray
            Vector on the left in Hessian evaluation.

        case_weights : ndarray
            Non-negative case weights

        Returns
        -------

        value : float
            Hessian evaluated at this pair of left and right vectors
        """

        if case_weights is None:
            case_weights = np.ones_like(natural_param)
        cw = case_weights
        if self.case_weights is not None:
            cw *= self.case_weights

        eta = natural_param # shorthand
        eta = self.apply_offset(eta)
        censoring = self.censoring

        H = np.zeros(eta.shape, np.float)

        return cox_hessian(H,
                           eta,
                           right_vector,
                           self._exp_buffer,
                           self._exp_accum,
                           self._expZ_accum,
                           self._outer_1st,
                           self._outer_2nd,
                           cw,
                           censoring,
                           self._ordering,
                           self._rankmin,
                           self._rankmax,
                           eta.shape[0])

    def get_data(self):
        return np.array([self.event_times, self.censoring]).T

    def set_data(self, data):
        event_times, censoring = np.asarray(data).T
        self.event_times, self.censoring = (np.asarray(event_times),
                                            np.asarray(censoring).astype(np.intp))

    data = property(get_data, set_data)

    def __copy__(self):
        event_times, censoring = self.event_times, self.censoring
        return cox_loglike(self.shape,
                           copy(event_times),
                           copy(censoring),
                           coef=self.coef,
                           offset=copy(self.offset),
                           quadratic=copy(self.quadratic),
                           initial=copy(self.coefs),
                           case_weights=copy(self.case_weights))

    def subsample(self, case_idx):
        """
        Create a saturated loss using a subsample of the data.
        Makes a copy of the loss and 
        multiplies case_weights by the indicator for
        `idx`.

        Parameters
        ----------

        idx : index
            Indices of np.arange(n) to keep.

        Returns
        -------

        subsample_loss : `smooth_atom`
            Loss after discarding all
            cases not in `idx.

        """
        loss_cp = copy(self)
        if loss_cp.case_weights is None:
            case_weights = loss_cp.case_weights = np.ones(self.shape[0])
        else:
            case_weights = loss_cp.case_weights

        idx_bool = np.zeros_like(case_weights, np.bool)
        idx_bool[case_idx] = 1

        case_weights *= idx_bool
        return loss_cp

    def latexify(self, var=None, idx=''):
        # a trick to get latex representation looking right
        # coxph should be written similar to logistic
        # composed with a linear transform (TODO)
        return smooth_atom.latexify(self, var=var, idx=idx)



have_sklearn = True
try:
    from sklearn.base import (BaseEstimator, 
                              RegressorMixin,
                              ClassifierMixin)
except ImportError:
    have_sklearn = False

if have_sklearn:

    from .gaussian import (sklearn_gaussian as gaussian,
                           sklearn_gaussian_lagrange as gaussian_lagrange)
    from .huber import (sklearn_huber as huber,
                        sklearn_huber_lagrange as huber_lagrange)
    from .cox import (sklearn_cox as cox,
                      sklearn_cox_lagrange as cox_lagrange)

    from .logistic import (sklearn_logistic as logistic,
                           sklearn_logistic_lagrange as logistic_lagrange,
                           sklearn_logistic_classifier as logistic_classifier,
                           sklearn_logistic_classifier_lagrange as logistic_classifier_lagrange)
    from .multinomial import (sklearn_multinomial as multinomial,
                              sklearn_multinomial_lagrange as multinomial_lagrange,
                              sklearn_multinomial_classifier as multinomial_classifier,
                              sklearn_multinomial_classifier_lagrange as multinomial_classifier_lagrange)
    from .probit import (sklearn_probit as probit,
                         sklearn_probit_lagrange as probit_lagrange,
                         sklearn_probit_classifier as probit_classifier,
                         sklearn_probit_classifier_lagrange as probit_classifier_lagrange)
    from .cloglog import (sklearn_cloglog as cloglog,
                          sklearn_cloglog_lagrange as cloglog_lagrange,
                          sklearn_cloglog_classifier as cloglog_classifier,
                          sklearn_cloglog_classifier_lagrange as cloglog_classifier_lagrange)
    from .huber_svm import (sklearn_huber_svm as huber_svm,
                            sklearn_huber_svm_lagrange as huber_svm_lagrange)
    



"""
This module contains the implementation of block norms, i.e.
l1/l*, linf/l* norms. These are used in multiresponse LASSOs.

"""
from __future__ import print_function, division, absolute_import

import warnings

import numpy as np

from . import seminorms
from ..identity_quadratic import identity_quadratic
from ..problems.composite import smooth_conjugate
from ..objdoctemplates import objective_doc_templater
from ..doctemplates import (doc_template_user, doc_template_provider)
from ..atoms import _work_out_conjugate
from .piecewise_linear import find_solution_piecewise_linear_c

# for the docstring, we need l1norm
l1norm = seminorms.l1norm

@objective_doc_templater()
class block_sum(seminorms.seminorm):

    objective_template = r"""\|%(var)s\|_{1,\|\cdot\|}"""
    objective_vars = seminorms.seminorm.objective_vars.copy()
    objective_vars['var'] = 'B'
    objective_vars['normklass'] = 'block_sum'
    objective_vars['dualnormklass'] = 'block_max'
    objective_vars['initargs'] = 'rr.l1norm, (5, 4)'
    objective_vars['shape'] = r'n \times p'

    def __init__(self, atom_cls, shape,
                 lagrange=None,
                 bound=None,
                 offset=None,
                 quadratic=None,
                 initial=None):

        seminorms.seminorm.__init__(self,
                            shape,
                            quadratic=quadratic,
                            offset=offset,
                            initial=initial,
                            lagrange=lagrange,
                            bound=bound)

        self.atom = atom_cls(shape[1:], lagrange=lagrange,
                             bound=bound,
                             offset=None,
                             quadratic=quadratic)

    @doc_template_user
    @doc_template_provider
    def seminorms(self, x, lagrange=None, check_feasibility=False):
        """
        Compute all seminorms in the block norm.
        """
        value = np.empty(self.shape[0])
        for i in range(self.shape[0]):
            value[i] = self.atom.seminorm(x[i], lagrange=lagrange,
                                          check_feasibility=False)
        return value

    @doc_template_user
    def seminorm(self, x, check_feasibility=False,
                 lagrange=None):
        x = x.reshape(self.shape)
        lagrange = seminorms.seminorm.seminorm(self, x, lagrange=lagrange,
                                 check_feasibility=check_feasibility)
        return lagrange * np.sum( \
            self.seminorms(x, check_feasibility=check_feasibility,
                           lagrange=1.))

    @doc_template_user
    def constraint(self, x):
        # XXX should we check feasibility here?
        x = x.reshape(self.shape)
        v = np.sum(self.seminorms(x, check_feasibility=False))
        if v <= self.bound * (1 + self.tol):
            return 0
        return np.inf

    @doc_template_user
    def lagrange_prox(self, x, lipschitz=1, lagrange=None):
        x = x.reshape(self.shape)
        lagrange = seminorms.seminorm.lagrange_prox(self, x, lipschitz, lagrange)
        v = np.empty(x.shape)
        for i in xrange(self.shape[0]):
            v[i] = self.atom.lagrange_prox(x[i], lipschitz=lipschitz,
                                           lagrange=lagrange)
        return v

    @doc_template_user
    def bound_prox(self, x, bound=None):
        x = x.reshape(self.shape)
        warnings.warn('bound_prox of block_sum requires a little thought -- should be like l1prox')
        return 0 * x

    @doc_template_user
    def get_lagrange(self):
        return seminorms.seminorm.get_lagrange(self)

    @doc_template_user
    def set_lagrange(self, lagrange):
        return seminorms.seminorm.set_lagrange(self)

    @doc_template_user
    def get_bound(self):
        return seminorms.seminorm.get_bound(self)

    @doc_template_user
    def set_bound(self, bound):
        return seminorms.seminorm.get_bound(self)

    @doc_template_user
    def get_conjugate(self):
        if self.quadratic.coef == 0:

            offset, outq = _work_out_conjugate(self.offset, 
                                               self.quadratic)

            cls = conjugate_block_pairs[self.__class__]
            conj_atom = self.atom.conjugate
            atom_cls = conj_atom.__class__

            atom = cls(atom_cls, 
                       self.shape, 
                       offset=offset,
                       lagrange=conj_atom.lagrange,
                       bound=conj_atom.bound,
                       quadratic=outq)

        else:
            atom = smooth_conjugate(self)
        self._conjugate = atom
        self._conjugate._conjugate = self
        return self._conjugate
    conjugate = property(get_conjugate)

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return seminorms.seminorm.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_bound(self):
        return seminorms.seminorm.get_bound(self)

    @doc_template_user
    def set_bound(self, bound):
        return seminorms.seminorm.set_bound(self, bound)

    @doc_template_user
    def get_lagrange(self):
        return seminorms.seminorm.get_lagrange(self)

    @doc_template_user
    def set_lagrange(self, lagrange):
        return seminorms.seminorm.set_lagrange(self, lagrange)

    @doc_template_user
    def get_dual(self):
        return seminorms.seminorm.dual(self)


@objective_doc_templater()
class block_max(block_sum):

    objective_template = r"""\|%(var)s\|_{\infty,\| \cdot\|}"""
    objective_vars = block_sum.objective_vars.copy()
    objective_vars['normklass'] = 'block_max'
    objective_vars['dualnormklass'] = 'block_sum'

    @doc_template_user
    def seminorm(self, x, lagrange=None, check_feasibility=False):
        x = x.reshape(self.shape)
        lagrange = seminorms.seminorm.seminorm(self, x, lagrange=lagrange,
                                 check_feasibility=check_feasibility)
        return lagrange * np.max(self.seminorms(x,  
                                                lagrange=1.,
                                                check_feasibility=check_feasibility))

    @doc_template_user
    def constraint(self, x, bound=None):
        x = x.reshape(self.shape)
        bound = seminorms.seminorm.constraint(self, x, bound=bound)
        # XXX should we check feasibility here?
        v = np.max(self.seminorms(x, lagrange=1., check_feasibility=False))
        if v <= self.bound * (1 + self.tol):
            return 0
        return np.inf

    @doc_template_user
    def lagrange_prox(self, x, lipschitz=1, lagrange=None):
        raise ValueError('lagrange_prox of block_max requires a little thought -- should be like l1prox')

    @doc_template_user
    def bound_prox(self, x, bound=None):
        x = x.reshape(self.shape)
        bound = seminorms.seminorm.bound_prox(self, x,
                                      bound=bound)
        v = np.empty(x.shape)
        for i in xrange(self.shape[0]):
            v[i] = self.atom.bound_prox(x[i], 
                                        bound=bound)
        return v

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return seminorms.seminorm.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_bound(self):
        return seminorms.seminorm.get_bound(self)

    @doc_template_user
    def set_bound(self, bound):
        return seminorms.seminorm.set_bound(self, bound)

    @doc_template_user
    def get_lagrange(self):
        return seminorms.seminorm.get_lagrange(self)

    @doc_template_user
    def set_lagrange(self, lagrange):
        return seminorms.seminorm.set_lagrange(self, lagrange)

    @doc_template_user
    def get_dual(self):
        return seminorms.seminorm.dual(self)

    @doc_template_user
    def get_conjugate(self):
        return block_sum.get_conjugate(self)


@objective_doc_templater()
class linf_l2(block_max):

    objective_template = r"""\|%(var)s\|_{\infty,2}"""
    objective_vars = block_sum.objective_vars.copy()
    objective_vars['normklass'] = 'linf_l2'
    objective_vars['dualnormklass'] = 'l1_l2'
    objective_vars['initargs'] = '(5,4)'

    def __init__(self, 
                 shape,
                 lagrange=None,
                 bound=None,
                 offset=None,
                 quadratic=None,
                 initial=None):
        block_max.__init__(self, 
                           seminorms.l2norm,
                           shape,
                           lagrange=lagrange,
                           bound=bound,
                           offset=offset,
                           quadratic=quadratic,
                           initial=initial)

    @doc_template_user
    def constraint(self, arg):
        arg = arg.reshape(self.shape)
        norm_max = np.sqrt((arg**2).sum(1)).max()
        if norm_max <= self.bound * (1 + self.tol):
            return 0
        return np.inf

    @doc_template_user
    def seminorm(self, arg, lagrange=None, check_feasibility=False):
        arg = arg.reshape(self.shape)
        lagrange = seminorms.seminorm.seminorm(self, arg, lagrange=lagrange,
                                 check_feasibility=check_feasibility)
        norm_max = np.sqrt((arg**2).sum(1)).max()
        return lagrange * norm_max

    @doc_template_user
    def bound_prox(self, arg, bound=None):
        arg = arg.reshape(self.shape)
        norm = np.sqrt((arg**2).sum(1))
        bound = seminorms.seminorm.bound_prox(self, arg,
                                              bound=bound)
        v = arg.copy()
        v[norm >= bound] *= bound / norm[norm >= bound][:,np.newaxis]
        return v

    @doc_template_user
    def lagrange_prox(self, arg,  lipschitz=1, lagrange=None):
        arg = arg.reshape(self.shape)
        norm = np.sqrt((arg**2).sum(1))
        lagrange = seminorms.seminorm.lagrange_prox(self, arg, lipschitz, lagrange)
        cut = find_solution_piecewise_linear_c(lagrange / lipschitz, 0, norm)
        if cut < np.inf:
            proj_factor = (norm - cut) * (norm > cut)
        else:
            proj_factor = arg
        factor = (norm - proj_factor) / norm
        v = arg.copy()
        v *= factor[:,np.newaxis]
        return v

    @doc_template_user
    def get_conjugate(self):

        if self.quadratic.coef == 0:
            offset, outq = _work_out_conjugate(self.offset, 
                                               self.quadratic)

            cls = conjugate_block_pairs[self.__class__]
            conj_atom = self.atom.conjugate

            atom = cls(self.shape, 
                       offset=offset,
                       lagrange=conj_atom.lagrange,
                       bound=conj_atom.bound,
                       quadratic=outq)

        else:
            atom = smooth_conjugate(self)
        self._conjugate = atom
        self._conjugate._conjugate = self
        return self._conjugate
    conjugate = property(get_conjugate)

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return seminorms.seminorm.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_bound(self):
        return seminorms.seminorm.get_bound(self)

    @doc_template_user
    def set_bound(self, bound):
        return seminorms.seminorm.set_bound(self, bound)

    @doc_template_user
    def get_lagrange(self):
        return seminorms.seminorm.get_lagrange(self)

    @doc_template_user
    def set_lagrange(self, lagrange):
        return seminorms.seminorm.set_lagrange(self, lagrange)

    @doc_template_user
    def get_dual(self):
        return seminorms.seminorm.dual(self)


@objective_doc_templater()
class linf_linf(linf_l2):

    objective_template = r"""\|%(var)s\|_{\infty,\infty}"""
    objective_vars = linf_l2.objective_vars.copy()
    objective_vars['normklass'] = 'linf_linf'
    objective_vars['dualnormklass'] = 'l1_l1'

    def __init__(self, shape,
                 lagrange=None,
                 bound=None,
                 offset=None,
                 quadratic=None,
                 initial=None):
        block_max.__init__(self, seminorms.supnorm,
                           shape,
                           lagrange=lagrange,
                           bound=bound,
                           offset=offset,
                           quadratic=quadratic,
                           initial=initial)

    @doc_template_user
    def constraint(self, arg):
        arg = arg.reshape(self.shape)
        norm_max = np.fabs(arg).max()
        if norm_max <= self.bound * (1 + self.tol):
            return 0
        return np.inf

    @doc_template_user
    def seminorm(self, arg, lagrange=None, check_feasibility=False):
        arg = arg.reshape(self.shape)
        lagrange = seminorms.seminorm.seminorm(self, arg, lagrange=lagrange,
                                 check_feasibility=check_feasibility)
        norm_max = np.fabs(arg).max()
        return lagrange * norm_max

    @doc_template_user
    def bound_prox(self, arg, bound=None):
        arg = arg.reshape(self.shape)
        bound = seminorms.seminorm.bound_prox(self, arg,
                                              bound=bound)
        return np.clip(arg, -bound, bound).reshape(self.shape)

    @doc_template_user
    def lagrange_prox(self, arg,  lipschitz=1, lagrange=None):
        lagrange = seminorms.seminorm.lagrange_prox(self, arg, lipschitz, lagrange)
        arg = np.asarray(arg, np.float).reshape(-1)
        absarg = np.fabs(arg)
        cut = find_solution_piecewise_linear_c(lagrange / lipschitz, 0, absarg)
        if cut < np.inf:
            proj = np.sign(arg) * (absarg - cut) * (absarg > cut)
        else:
            proj = arg
        return (arg - proj).reshape(self.shape)

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return seminorms.seminorm.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_bound(self):
        return seminorms.seminorm.get_bound(self)

    @doc_template_user
    def set_bound(self, bound):
        return seminorms.seminorm.set_bound(self, bound)

    @doc_template_user
    def get_lagrange(self):
        return seminorms.seminorm.get_lagrange(self)

    @doc_template_user
    def set_lagrange(self, lagrange):
        return seminorms.seminorm.set_lagrange(self, lagrange)

    @doc_template_user
    def get_dual(self):
        return seminorms.seminorm.dual(self)

    @doc_template_user
    def get_dual(self):
        return seminorms.seminorm.dual(self)

    @doc_template_user
    def get_conjugate(self):
        return linf_l2.get_conjugate(self)


@objective_doc_templater()
class l1_l2(block_sum):

    objective_template = r"""\|%(var)s\|_{1,2}"""
    objective_vars = linf_l2.objective_vars.copy()
    objective_vars['normklass'] = 'l1_l2'
    objective_vars['dualnormklass'] = 'linf_l2'

    def __init__(self, shape,
                 lagrange=None,
                 bound=None,
                 offset=None,
                 quadratic=None,
                 initial=None):
        block_sum.__init__(self, seminorms.l2norm,
                           shape,
                           lagrange=lagrange,
                           bound=bound,
                           offset=offset,
                           quadratic=quadratic,
                           initial=initial)

    @doc_template_user
    def lagrange_prox(self, arg, lipschitz=1, lagrange=None):
        arg = arg.reshape(self.shape)
        lagrange = seminorms.seminorm.lagrange_prox(self, arg, lipschitz, lagrange)
        norm = np.sqrt((arg**2).sum(1))
        mult = np.maximum(norm - lagrange / lipschitz, 0) / norm
        return arg * mult[:, np.newaxis]

    @doc_template_user
    def bound_prox(self, arg, bound=None):
        arg = arg.reshape(self.shape)
        norm = np.sqrt((arg**2).sum(1))
        bound = seminorms.seminorm.bound_prox(self, arg,
                                              bound=bound)
        cut = find_solution_piecewise_linear_c(bound, 0, norm)
        if cut < np.inf:
            proj_factor = (norm - cut) * (norm > cut)
        else:
            proj_factor = arg
        factor = proj_factor / norm
        v = arg.copy()
        v *= factor[:,np.newaxis]
        return v

    @doc_template_user
    def constraint(self, x):
        x = x.reshape(self.shape)
        norm_sum = np.sqrt((x**2).sum(1)).sum()
        if norm_sum <= self.bound * (1 + self.tol):
            return 0
        return np.inf

    @doc_template_user
    def seminorm(self, x, lagrange=None, check_feasibility=False):
        x = x.reshape(self.shape)
        lagrange = seminorms.seminorm.seminorm(self, x, lagrange=lagrange,
                                 check_feasibility=check_feasibility)
        norm_sum = np.sum(np.sqrt((x**2).sum(1)))
        return lagrange * norm_sum

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return seminorms.seminorm.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_bound(self):
        return seminorms.seminorm.get_bound(self)

    @doc_template_user
    def set_bound(self, bound):
        return seminorms.seminorm.set_bound(self, bound)

    @doc_template_user
    def get_lagrange(self):
        return seminorms.seminorm.get_lagrange(self)

    @doc_template_user
    def set_lagrange(self, lagrange):
        return seminorms.seminorm.set_lagrange(self, lagrange)

    @doc_template_user
    def get_dual(self):
        return seminorms.seminorm.dual(self)

    @doc_template_user
    def get_dual(self):
        return seminorms.seminorm.dual(self)

    @doc_template_user
    def get_conjugate(self):
        if self.quadratic.coef == 0:
            offset, outq = _work_out_conjugate(self.offset, 
                                               self.quadratic)

            cls = conjugate_block_pairs[self.__class__]
            conj_atom = self.atom.conjugate

            atom = cls(self.shape, 
                       offset=offset,
                       lagrange=conj_atom.lagrange,
                       bound=conj_atom.bound,
                       quadratic=outq)

        else:
            atom = smooth_conjugate(self)
        self._conjugate = atom
        self._conjugate._conjugate = self
        return self._conjugate
    conjugate = property(get_conjugate)

@objective_doc_templater()
class l1_l1(l1_l2):

    objective_template = r"""\|%(var)s\|_{1,1}"""
    objective_vars = l1_l2.objective_vars.copy()
    objective_vars['normklass'] = 'l1_l1'
    objective_vars['dualnormklass'] = 'linf_linf'

    def __init__(self, shape,
                 lagrange=None,
                 bound=None,
                 offset=None,
                 quadratic=None,
                 initial=None):
        block_sum.__init__(self, seminorms.l1norm,
                           shape,
                           lagrange=lagrange,
                           bound=bound,
                           offset=offset,
                           quadratic=quadratic,
                           initial=initial)

    @doc_template_user
    def lagrange_prox(self, arg, lipschitz=1, lagrange=None):
        arg = arg.reshape(self.shape)
        lagrange = seminorms.seminorm.lagrange_prox(self, arg, lipschitz, lagrange)
        norm = np.fabs(arg)
        return (np.maximum(norm - lagrange / lipschitz, 0) * np.sign(arg)).reshape(self.shape)

    @doc_template_user
    def constraint(self, arg):
        arg = arg.reshape(self.shape)
        norm_sum = np.fabs(arg).sum()
        if norm_sum <= self.bound * (1 + self.tol):
            return 0
        return np.inf

    @doc_template_user
    def seminorm(self, arg, lagrange=None, check_feasibility=False):
        arg = arg.reshape(self.shape)
        lagrange = seminorms.seminorm.seminorm(self, arg, lagrange=lagrange,
                                 check_feasibility=check_feasibility)
        norm_sum = np.fabs(arg).sum()
        return lagrange * norm_sum

    @doc_template_user
    def bound_prox(self, arg, bound=None):
        bound = seminorms.seminorm.bound_prox(self, arg, bound)
        arg = np.asarray(arg, np.float).reshape(-1)
        absarg = np.fabs(arg)
        cut = find_solution_piecewise_linear_c(bound, 0, absarg)
        if cut < np.inf:
            value = np.sign(arg) * (absarg - cut) * (absarg > cut)
        return value.reshape(self.shape)

    @doc_template_user
    def proximal(self, quadratic, prox_control=None):
        return seminorms.seminorm.proximal(self, quadratic, prox_control)

    @doc_template_user
    def get_bound(self):
        return seminorms.seminorm.get_bound(self)

    @doc_template_user
    def set_bound(self, bound):
        return seminorms.seminorm.set_bound(self, bound)

    @doc_template_user
    def get_lagrange(self):
        return seminorms.seminorm.get_lagrange(self)

    @doc_template_user
    def set_lagrange(self, lagrange):
        return seminorms.seminorm.set_lagrange(self, lagrange)

    @doc_template_user
    def get_dual(self):
        return seminorms.seminorm.dual(self)

    @doc_template_user
    def get_dual(self):
        return seminorms.seminorm.dual(self)

    @doc_template_user
    def get_conjugate(self):
        return l1_l2.get_conjugate(self)

conjugate_block_pairs = {}
for n1, n2 in [(block_max, block_sum),
               (l1_l2, linf_l2),
               (l1_l1, linf_linf)
               ]:
    conjugate_block_pairs[n1] = n2
    conjugate_block_pairs[n2] = n1



import itertools

import numpy as np

import regreg.api as rr
import regreg.atoms.simplex as S
from regreg.tests.decorators import set_seed_for_test

import nose.tools as nt

from .test_cones import Solver, ConeSolverFactory


class SimplexSolverFactory(ConeSolverFactory):

    FISTA_choices = [True]
    L_choices = [1, 0.3]
    coef_stop_choices = [False]
    shape = 5
    
    def __init__(self, klass):
        self.klass = klass

    def __iter__(self):
        for offset, FISTA, coef_stop, L, q in itertools.product(self.offset_choices,
                                                                self.FISTA_choices,
                                                                self.coef_stop_choices,
                                                                self.L_choices,
                                                                self.quadratic_choices):
            self.FISTA = FISTA
            self.coef_stop = coef_stop
            self.L = L

            atom = self.klass(self.shape)

            # make sure certain lines of code are tested
            assert(atom == atom)
            atom.latexify(), atom.dual, atom.conjugate

            if q: 
                atom.quadratic = rr.identity_quadratic(0,0,np.random.standard_normal(atom.shape)*0.02)

            if offset:
                atom.offset = 0.02 * np.random.standard_normal(atom.shape)

            solver = SimplexSolver(atom,
                                   interactive=self.interactive, 
                                   coef_stop=coef_stop,
                                   FISTA=FISTA,
                                   L=L)
            yield solver

class SimplexSolver(Solver):

    def all_tests(self):
        for group in [self.test_duality_of_projections,
                      self.test_simple_problem,
                      self.test_separable,
                      self.test_dual_problem,
                      self.test_container,
                      self.test_simple_problem_nonsmooth
                      ]:
            for t in group():
                yield t

@set_seed_for_test()
@np.testing.dec.slow
def test_proximal_maps():
    for klass in sorted(S.conjugate_simplex_pairs.keys(), key=str):
        factory = SimplexSolverFactory(klass)
        for solver in factory:
            for t in solver.all_tests():
                yield t



import numpy as np

from .. import lasso, strong_rules, warm_start
from ..basil import basil_inner_loop, basil
from ...tests.decorators import set_seed_for_test

@set_seed_for_test()
def test_path(n=200,p=50):
    '''
    run a basic path algorithm

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    lasso1 = lasso.gaussian(X, Y, np.ones(X.shape[1]))
    lagrange_sequence = lasso.default_lagrange_sequence(lasso1.penalty,
                                                        lasso1.grad_solution,
                                                        nstep=23) # initialized at "null" model
    sol1 = strong_rules(lasso1, 
                        lagrange_sequence, 
                        (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                        inner_tol=1.e-5)
    beta1 = sol1['beta']

@set_seed_for_test()
def test_warm_start(n=200,p=50):
    '''
    compare full problem with warm start to strong rules path

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    lasso1 = lasso.gaussian(X, Y, np.ones(X.shape[1]))
    lagrange_sequence = lasso.default_lagrange_sequence(lasso1.penalty,
                                                        lasso1.grad_solution,
                                                        nstep=23) # initialized at "null" model
    sol1 = strong_rules(lasso1, 
                        lagrange_sequence, 
                        (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                        inner_tol=1.e-14)
    beta1 = sol1['beta']

    sol2 = warm_start(lasso1, 
                      lagrange_sequence, 
                      (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                      inner_tol=1.e-14)
    beta2 = sol2['beta']

    np.testing.assert_allclose(beta1, beta2, rtol=1.e-4)

@set_seed_for_test()
def test_warm_start_enet(n=200,p=50):
    '''
    compare full problem with warm start to strong rules path
    including elastic net

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    enet = np.ones(X.shape[1])
    enet[4:8] = 0

    lasso1 = lasso.gaussian(X, 
                            Y, 
                            np.ones(X.shape[1]),
                            alpha=0.5,
                            elastic_net_param=enet)

    lagrange_sequence = lasso.default_lagrange_sequence(lasso1.penalty,
                                                        lasso1.grad_solution,
                                                        nstep=23,
                                                        alpha=lasso1.alpha) # initialized at "null" model
    sol1 = strong_rules(lasso1, 
                        lagrange_sequence, 
                        (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                        inner_tol=1.e-14)
    beta1 = sol1['beta']

    sol2 = warm_start(lasso1, 
                      lagrange_sequence, 
                      (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                      inner_tol=1.e-14)
    beta2 = sol2['beta']

    assert(np.linalg.norm(beta1 - beta2)/np.linalg.norm(beta1) < 1.e-4)
    
   
@set_seed_for_test()
def test_path_subsample(n=200,p=50):
    '''
    compare a subsample path to the full path on subsampled data

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    cases = range(n//2)
    lasso1 = lasso.gaussian(X, Y, np.ones(X.shape[1]))
    lasso1_sub = lasso1.subsample(cases)
    lagrange_sequence = lasso.default_lagrange_sequence(lasso1_sub.penalty,
                                                        lasso1_sub.grad_solution,
                                                        nstep=23) # initialized at "null" model
    sol1 = strong_rules(lasso1_sub, 
                        lagrange_sequence, 
                        (lasso1_sub.solution.copy(), lasso1_sub.grad_solution.copy()),
                        inner_tol=1.e-10)
    beta1 = sol1['beta']

    lasso2 = lasso.gaussian(X[cases], Y[cases], np.ones(X.shape[1]))
    sol2 = strong_rules(lasso2, 
                        lagrange_sequence, 
                        (lasso2.solution.copy(), lasso2.grad_solution.copy()),
                        inner_tol=1.e-10)
    beta2 = sol2['beta']

    yield np.testing.assert_allclose, beta1, beta2, 1.e-3

    # check that the subsample did not change the original
    # path object

    lasso3 = lasso.gaussian(X, Y, np.ones(X.shape[1]))
    beta3 = strong_rules(lasso1, 
                         lagrange_sequence, 
                         (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                         inner_tol=1.e-10)['beta']
    beta4 = strong_rules(lasso3, 
                         lagrange_sequence, 
                         (lasso3.solution.copy(), lasso3.grad_solution.copy()),
                         inner_tol=1.e-10)['beta']

    yield np.testing.assert_allclose, beta3, beta4, 1.e-3

@set_seed_for_test()
def test_unpenalized(n=200, p=50):
    '''
    run a basic path algorithm with some unpenalized variables

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    weights = np.ones(X.shape[1])
    weights[0:2] = 0
    lasso1 = lasso.gaussian(X, 
                            Y, 
                            weights)
    lagrange_sequence = lasso.default_lagrange_sequence(lasso1.penalty,
                                                        lasso1.grad_solution,
                                                        nstep=23) # initialized at "null" model
    sol1 = strong_rules(lasso1, 
                        lagrange_sequence, 
                        (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                        inner_tol=1.e-5)
    beta1 = sol1['beta']

@set_seed_for_test()
def test_elastic_net(n=200, p=50):
    '''
    run a basic elastic net path algorithm

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    weights = np.ones(X.shape[1])
    lasso1 = lasso.gaussian(X, 
                            Y, 
                            weights,
                            alpha=0.5,
                            elastic_net_param=np.ones(X.shape[1]))
    lagrange_sequence = lasso.default_lagrange_sequence(lasso1.penalty,
                                                        lasso1.grad_solution,
                                                        nstep=23,
                                                        alpha=lasso1.alpha) # initialized at "null" model
    sol1 = strong_rules(lasso1,
                        lagrange_sequence, 
                        (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                        inner_tol=1.e-5)
    beta1 = sol1['beta']

@set_seed_for_test()
def test_elastic_net_unpenalized(n=200, p=50):
    '''
    run a basic elastic net path algorithm with unpenalized
    

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    enet = np.ones(X.shape[1])
    enet[0] = 0
    weights = np.ones(X.shape[1])
    weights[0] = 0

    lasso1 = lasso.gaussian(X, 
                            Y, 
                            weights,
                            alpha=0.5,
                            elastic_net_param=enet)
    lagrange_sequence = lasso.default_lagrange_sequence(lasso1.penalty,
                                                        lasso1.grad_solution,
                                                        nstep=23,
                                                        alpha=lasso1.alpha) # initialized at "null" model
    sol1 = strong_rules(lasso1, 
                        lagrange_sequence, 
                        (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                        inner_tol=1.e-5)
    beta1 = sol1['beta']

@set_seed_for_test()
def test_basil_inner_loop(n=1000,p=600):
    '''
    test one run of the BASIL inner loop

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5] / np.sqrt(n)
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    lasso1 = lasso.gaussian(X, Y, np.ones(X.shape[1]))
    lagrange_sequence = lasso.default_lagrange_sequence(lasso1.penalty,
                                                        lasso1.grad_solution,
                                                        nstep=100) # initialized at "null" model
    sol1 = basil_inner_loop(lasso1, 
                            lagrange_sequence[:50], 
                            (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                            inner_tol=1.e-14,
                            step_nvar=10)
    lagrange1, beta1, grad1, active1 = sol1
    print(np.array(beta1).shape, 'chunk of path')
    print(active1, 'active')

@set_seed_for_test()
def test_basil(n=1000,p=600):
    '''
    test BASIL

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5] / np.sqrt(n)
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    lasso1 = lasso.gaussian(X, 
                            Y, 
                            np.ones(X.shape[1]))
    lagrange_sequence = lasso.default_lagrange_sequence(lasso1.penalty,
                                                        lasso1.grad_solution,
                                                        nstep=100) # initialized at "null" model
    sol1 = basil(lasso1, 
                 lagrange_sequence, 
                 (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                 inner_tol=1.e-14,
                 step_nvar=10,
                 step_lagrange=20)

    lasso2 = lasso.gaussian(X, 
                            Y, 
                            np.ones(X.shape[1]))
    sol2 = warm_start(lasso2, 
                      lagrange_sequence, 
                      (lasso2.solution.copy(), lasso2.grad_solution.copy()),
                      inner_tol=1.e-14)['beta']

    assert(np.linalg.norm(sol1 - sol2) / np.linalg.norm(sol2) < 1.e-4)

@set_seed_for_test()
def test_basil_unpenalized(n=500,p=200):
    '''
    test BASIL

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5] / np.sqrt(n)
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    W = np.ones(X.shape[1])
    W[2:4] = 0
    lasso1 = lasso.gaussian(X, 
                            Y, 
                            W)
    lagrange_sequence = lasso.default_lagrange_sequence(lasso1.penalty,
                                                        lasso1.grad_solution,
                                                        lagrange_proportion=0.4,
                                                        nstep=20) # initialized at "null" model
    sol1 = basil(lasso1, 
                 lagrange_sequence, 
                 (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                 inner_tol=1.e-14,
                 step_nvar=10,
                 step_lagrange=20)

    lasso2 = lasso.gaussian(X, 
                            Y, 
                            W)
    sol2 = warm_start(lasso2, 
                      lagrange_sequence, 
                      (lasso2.solution.copy(), lasso2.grad_solution.copy()),
                      inner_tol=1.e-14)['beta']

    assert(np.linalg.norm(sol1 - sol2) / np.linalg.norm(sol2) < 1.e-4)

@set_seed_for_test()
def test_basil_enet(n=500,p=200):
    '''
    test BASIL w/enet

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5] / np.sqrt(n)
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    enet = np.ones(X.shape[1])
    enet[4:8] = 0

    lasso1 = lasso.gaussian(X, 
                            Y, 
                            np.ones(X.shape[1]),
                            alpha=0.5,
                            elastic_net_param=enet)

    lagrange_sequence = lasso.default_lagrange_sequence(lasso1.penalty,
                                                        lasso1.grad_solution,
                                                        lagrange_proportion=0.5,
                                                        nstep=50,
                                                        alpha=lasso1.alpha) # initialized at "null" model
    sol1 = basil(lasso1, 
                 lagrange_sequence, 
                 (lasso1.solution.copy(), lasso1.grad_solution.copy()),
                 inner_tol=1.e-14,
                 step_nvar=10,
                 step_lagrange=20)

    lasso2 = lasso.gaussian(X, 
                            Y, 
                            np.ones(X.shape[1]),
                            alpha=0.5,
                            elastic_net_param=enet)

    sol2 = warm_start(lasso2, 
                      lagrange_sequence, 
                      (lasso2.solution.copy(), lasso2.grad_solution.copy()),
                      inner_tol=1.e-14)['beta']

    lagrange_sequence2 = lasso.default_lagrange_sequence(lasso2.penalty,
                                                         lasso2.grad_solution,
                                                         lagrange_proportion=0.5,
                                                         nstep=50,
                                                         alpha=lasso2.alpha) # initialized at "null" model

    rel_errors = np.array([np.linalg.norm(sol1[i]-sol2[i]) / np.linalg.norm(sol1[i])
                           for i in range(1, sol1.shape[0])])

    assert(np.percentile(rel_errors, 90) < 1.e-4)




import numpy as np

have_sklearn = True
try:
    from .base import (sklearn_regression,
                       sklearn_regression_lagrange)
except ImportError:
    have_sklearn = False
from ..smooth.glm import (glm, 
                          gaussian_loglike)

if have_sklearn:
    class sklearn_gaussian(sklearn_regression):

        """

        A simple regression mixin for sklearn
        that allows any atom to be used as a regularizer.

        """

        def _loglike_factory(self, X, y):
            response, case_weights_, offset_ = self._check_y_arg(y)
            return glm.gaussian(X, 
                                response, 
                                case_weights=case_weights_,
                                coef=self.coef,
                                saturated_offset=offset_)

        def _saturated_score(self,
                             predictions,
                             response,
                             case_weights=None):

            loss = lambda yhat: gaussian_loglike(response.shape,
                                                 response,
                                                 case_weights=case_weights).smooth_objective(yhat, 'func')

            # factor of 2 to form proper deviance (default is negative log-likelihood,
            # while deviance is 2 * negative log-likelihood
            # negative sign is to align with sklearn's maximizing a score with grid search

            if self.score_method == 'deviance':
                return -2 * loss(predictions)
            elif self.score_method == 'mean_deviance':
                return -2 * loss(predictions) / predictions.shape[0]
            elif self.score_method == 'R2':
                SSE = 2 * loss(predictions)
                SST = 2 * loss(response.mean() * np.ones_like(response))
                return 1 - SSE / SST
            else:
                return np.nan

    class sklearn_gaussian_lagrange(sklearn_regression_lagrange, sklearn_gaussian):

        """

        A simple regression mixin for sklearn
        that allows any atom to be used as a regularizer.

        """
        pass




import itertools
import numpy as np
import nose.tools as nt

import regreg.api as rr
from regreg.atoms.sparse_group_lasso import (sparse_group_lasso,
                                             sparse_group_lasso_dual,
                                             inside_set,
                                             gauge_function_dual)
from regreg.tests.decorators import set_seed_for_test

from .test_seminorms import Solver, SolverFactory, all_close

def test_l1norm_equivalent():
    """
    with equal weights the prox is the same as l1 norm
    """
    pen1 = sparse_group_lasso([1,1,2,2,2], 
                              np.ones(5), 
                              weights={1:0, 2:0},
                              lagrange=0.4)
    pen2 = rr.l1norm(5, lagrange=0.4)

    Z = np.array([3,2,4,6,7])
    np.testing.assert_allclose(pen1.lagrange_prox(Z), pen2.lagrange_prox(Z))

    Z = np.random.standard_normal(5) * 100
    np.testing.assert_allclose(pen1.lagrange_prox(Z), pen2.lagrange_prox(Z))

    dual1 = pen1.conjugate
    dual2 = pen2.conjugate

    np.testing.assert_allclose(Z, pen1.lagrange_prox(Z) + dual1.bound_prox(Z))
    np.testing.assert_allclose(dual1.bound_prox(Z), dual2.bound_prox(Z))

def test_group_lasso_equivalent():
    """
    with 0 as lasso weights should be group lasso
    """
    pen1 = sparse_group_lasso([1,1,2,2,2], 
                              np.zeros(5), 
                              weights={1:0.2, 2:0.1},
                              lagrange=0.4)
    pen2 = rr.group_lasso([1,1,2,2,2], {1:0.2, 2:0.1}, lagrange=0.4)

    Z = np.array([3,2,4,6,7])
    np.testing.assert_allclose(pen1.lagrange_prox(Z), pen2.lagrange_prox(Z))

    Z = np.random.standard_normal(5) * 100
    np.testing.assert_allclose(pen1.lagrange_prox(Z), pen2.lagrange_prox(Z))

    dual1 = pen1.conjugate
    dual2 = pen2.conjugate

    np.testing.assert_allclose(Z, pen1.lagrange_prox(Z) + dual1.bound_prox(Z))
    np.testing.assert_allclose(dual1.bound_prox(Z), dual2.bound_prox(Z))

def test_prox():

    pen = sparse_group_lasso([1,1,2,2,2], 
                             np.ones(5), 
                             weights={1:1, 2:1},
                             lagrange=1)

    arg = np.array([5,7,3,5,12.])

    prox_arg = pen.lagrange_prox(arg)
    
    soft_thresh = arg - 1.
    final_soln = np.zeros_like(soft_thresh)

    norm1 = np.linalg.norm(soft_thresh[:2])
    factor1 = (norm1 - 1) / norm1
    final_soln[:2] = soft_thresh[:2] * factor1

    norm2 = np.linalg.norm(soft_thresh[2:])
    factor2 = (norm2 - 1) / norm2
    final_soln[2:] = soft_thresh[2:] * factor2

    print(factor1, factor2)
    np.testing.assert_allclose(prox_arg, final_soln)
    
def test_inside_set():
    """
    with 0 as lasso weights should be group lasso
    """
    pen = sparse_group_lasso([1,1,2,2,2], 
                             np.ones(5), 
                             weights={1:0.2, 2:0.1},
                             lagrange=0.4)
    point = np.zeros(5)
    assert inside_set(pen, point)

    Z = np.random.standard_normal(5) * 20
    proxZ = pen.lagrange_prox(Z)
    assert inside_set(pen, Z - proxZ)  # its gauge norm is 0.4, larger than 1

    assert np.fabs(gauge_function_dual(pen, Z - proxZ) - 0.4) < 1.e-4

    pen2 = sparse_group_lasso([1,1,2,2,2], 
                             np.ones(5), 
                             weights={1:0.2, 2:0.1},
                             lagrange=1.2)
    point = np.zeros(5)
    assert inside_set(pen2, point) 

    Z = np.random.standard_normal(5) * 20
    proxZ = pen2.lagrange_prox(Z)
    print(Z, proxZ)
    assert np.fabs(gauge_function_dual(pen2, Z - proxZ) - 1.2) < 1.e-4
    assert not inside_set(pen, Z - proxZ) # its gauge norm is 1.2, larger than 1

class SparseGroupSolverFactory(SolverFactory):

    groups = [[0]*12 + [1]*8]
    weights = [{0:0.1,1:0.04}]
    lasso_weights = [np.linspace(0, 1, 20), np.zeros(20)]
    FISTA_choices = [True]
    L_choices = [0.3]
    coef_stop_choices = [False]

    def __init__(self, klass, mode):
        self.klass = klass
        self.mode = mode

    def __iter__(self):
        pen_choices = itertools.product(self.weights,
                                        self.lasso_weights,
                                        self.groups)
        for offset, FISTA, coef_stop, L, q, pen in itertools.product(self.offset_choices,
                                                                     self.FISTA_choices,
                                                                     self.coef_stop_choices,
                                                                     self.L_choices,
                                                                     self.quadratic_choices,
                                                                     pen_choices):
            self.FISTA = FISTA
            self.coef_stop = coef_stop
            self.L = L

            w, l, g = pen
            if self.mode == 'lagrange':
                atom = self.klass(g, l, weights=w, lagrange=self.lagrange)
            else:
                atom = self.klass(g, l, weights=w, bound=self.bound)

            if q: 
                atom.quadratic = rr.identity_quadratic(0, 0, np.random.standard_normal(atom.shape)*0.02)

            if offset:
                atom.offset = 0.02 * np.random.standard_normal(atom.shape)

            solver = Solver(atom, interactive=self.interactive, 
                            coef_stop=coef_stop,
                            FISTA=FISTA,
                            L=L)
            yield solver

@set_seed_for_test()
@np.testing.dec.slow
def test_proximal_maps():
    for klass, mode in zip([sparse_group_lasso, 
                            sparse_group_lasso_dual], 
                           ['lagrange', 'bound']):
        factory = SparseGroupSolverFactory(klass, mode)
        for solver in factory:
            for t in solver.all_tests():
                yield t
        factory = SparseGroupSolverFactory(klass, mode)
        for solver in factory:
            for t in solver.all_tests():
                yield t





"""
Simple ADMM implementation
"""

import numpy as np
from ..smooth import sum as smooth_sum
from ..smooth.quadratic import quadratic_loss
from ..affine import aslinear, astransform
from ..identity_quadratic import identity_quadratic

class admm_problem(object):

    """
    Objective function is the loss plus the atom composed with the transform.

    .. math::

        \ell(x) + {\cal P}(Dx + \alpha)

    Our transform is a map $\beta \mapsto D\beta + \alpha, and we introduce new variables
    $z=D\beta+\alpha$ so our affine constraint is $Dx-z=-\alpha$.

    In ADMM slides p.14 of https://stanford.edu/class/ee364b/lectures/admm_slides.pdf
    we have $A,B,c=D,-I,-\alpha$.

    """

    def __init__(self, 
                 loss, 
                 atom, 
                 transform, 
                 augmented_param, 
                 quadratic=None,
                 fit_args={}):

        (self.loss,
         self.atom,
         self.transform,
         self.augmented_param,
         self.quadratic) = (loss,
                            atom,
                            astransform(transform),
                            augmented_param,
                            quadratic)

        self.loss_coefs = self.loss.coefs                       # x in ADMM notes
        self.dual_coefs = np.zeros(self.transform.output_shape) # y in ADMM notes
        self.atom_coefs = np.zeros(self.transform.output_shape) # z in ADMM notes
        self.augmented_param = augmented_param                  # rho in ADMM notes

        self.linear_transform = aslinear(self.transform)                # D
        qloss = quadratic_loss.squared_transform(self.linear_transform) # x^TD^TDx / 2
        qloss.coef *= self.augmented_param                              # scale by rho
        self.augmented_loss = smooth_sum([self.loss,
                                          qloss])

        self.fit_args = fit_args # for the smooth_sum FISTA run

        assert (self.loss.shape == self.transform.output_shape)

    def update_loss_coefs(self):
        """
        Minimize over x: self.loss(x) + rho/2 x^TD^TDx + x^T D^T(y -rho * (z - \alpha))
        """
        y, z, rho, alpha = (self.dual_coefs,
                            self.atom_coefs,
                            self.augmented_param,
                            self.transform.affine_offset)
        if alpha is None:
            alpha = 0
        linear_term = self.linear_transform.dot(y - rho * (z - alpha))
        self.loss_coefs[:] = self.augmented_loss.solve(quadratic=identity_quadratic(0, 0, linear_term, 0),
                                                       **self.fit_args)

    def update_atom_coefs(self):
        """
        Minimize over z: atom(z) + rho/2 \|z-Dx-\alpha\|^2_2 - z^Ty
        """
        rho = self.augmented_param
        center_term = self.transform.affine_map(self.loss_coefs)
        linear_term = -self.dual_coefs
        Q = identity_quadratic(rho, center_term, linear_term, 0)
        self.atom_coefs[:] = self.atom.proximal(Q)

    def update_dual_coefs(self):
        """
        Dual update
        """
        rho = self.augmented_param
        self.dual_coefs[:] += rho * (self.transform.affine_map(self.loss_coefs) - self.atom_coefs)

    def solve(self, niter=20):
        for _ in range(niter):
            self.update_loss_coefs()
            self.update_atom_coefs()
            self.update_dual_coefs()




from __future__ import print_function, division, absolute_import

from warnings import warn
from copy import copy
import gc

import numpy as np
import numpy.linalg as npl

from scipy.stats import rankdata

from . import (subsample_columns, 
               grouped_path)
from ..affine import astransform
from ..affine.block_maps import block_columns
from ..smooth import (mglm, 
                      affine_smooth, 
                      glm,
                      sum as smooth_sum)
from ..smooth.quadratic import quadratic_loss
from ..problems.simple import simple_problem
from ..identity_quadratic import identity_quadratic as iq
from ..atoms.sparse_group_block import sparse_group_block
from ..atoms.sparse_group_lasso import (_gauge_function_dual_strong, 
                                        _inside_set_strong)
from .group_lasso import group_lasso_path, default_lagrange_sequence

class sparse_group_common_path(group_lasso_path):

    BIG = 1e12 # lagrange parameter for finding null solution

    """
    Responses here share a common X, e.g. multinomial regression.
    """

    def __init__(self, 
                 saturated_loss, # shape (n,q) -- multiresponse
                 X, 
                 l1_penalty,
                 l2_penalty,
                 elastic_net_param=None,
                 alpha=1.,  # elastic net mixing -- 1 is LASSO
                 l1_alpha=None): # this is \alpha of SGL paper,
                                 # if not None use convex comb of
                                 # implied l1,l2 penalties

        self.saturated_loss = saturated_loss
        self._X = np.asarray(X)
        self.X = astransform(X)
        self.X.output_shape = self.saturated_loss.shape
        if len(self.X.input_shape) == 1:                         # assumes X is matrix like, 
            self.X.input_shape = (self.X.input_shape +           # but its output should be same shape
                                  self.saturated_loss.shape[1:]) # as self.saturated_loss.shape

        self.l1_penalty = l1_penalty
        self.l2_penalty = l2_penalty
        if l1_alpha is not None:
            self.l1_penalty *= l1_alpha
            self.l2_penalty *= (1 - l1_alpha)

        # the penalty parameters

        self.alpha = alpha
        self.penalty = sparse_group_block(self.X.input_shape,
                                          l1_penalty, 
                                          l2_penalty, 
                                          lagrange=1)
        self.group_shape = (self.penalty.shape[0],)
        self.shape = self.penalty.shape

        # elastic net part
        if elastic_net_param is None:
            elastic_net_param = np.ones(self.shape)
        self.elastic_net_param = elastic_net_param

        unpenalized_groups, unpenalized_idx = self.unpenalized # [], []
        self.solution = np.zeros(self.penalty.shape)

        self._unpenalized_vars = []
        self._unpenalized_idx = unpenalized_idx
        self._penalized_vars = np.ones(self.shape, np.bool)

        if np.any(unpenalized_idx):
            (self.final_step, 
             null_grad, 
             null_soln,
             null_linpred,
             _) = self.solve_subproblem(unpenalized_groups,
                                        self.BIG,
                                        tol=1.e-8)
            self.linear_predictor = null_linpred
            self.solution[self._unpenalized_vars] = null_soln
        else:
            self.linear_predictor = np.zeros(self.saturated_loss.shape)

        if np.any(self.elastic_net_param[self._unpenalized_vars]):
            warn('convention is that unpenalized parameters with have no Lagrange parameter in front '
                 'of their ridge term so that lambda_max is easily computed')

        self.grad_solution = (self.full_gradient(self.saturated_loss, 
                                                 self.linear_predictor) + self.enet_grad(self.solution, 
                                                                                         self._penalized_vars,
                                                                                         1))

    # methods potentially overwritten in subclasses for I/O considerations

    def enet_loss(self,
                  lagrange,
                  candidate_groups=None):
        return _restricted_elastic_net(self.elastic_net_param, 
                                       self._penalized_vars,
                                       lagrange,
                                       self.alpha,
                                       candidate_groups)

    def subsample_columns(self, 
                          X,  # ignored
                          columns):
        """
        Extract columns of self.Xs into ndarray or
        Argument `X` is ignored here.
        """
        old_shape = self.X.output_shape
        # because it is common we know that self._X is its matrix representation
        Xsub = astransform(group_lasso_path.subsample_columns(self,
                                                              self._X,
                                                              columns))
        mask = np.zeros(self.penalty.shape[0], np.bool)
        mask[columns] = 1
        Xsub.input_shape = (mask.sum(),
                         self.penalty.shape[1])
        Xsub.output_shape = old_shape
        return Xsub

    def subsample(self,
                  case_idx):
        '''

        Create a new path, by subsampling
        cases of `self.saturated_loss`.

        Case weights are computed
        with `self.saturated_loss.subsample`.

        Parameters
        ----------

        case_idx : index
            An index-like object used 
            to specify which cases to include
            in the subsample.

        Returns
        -------

        subsample_path : path object
            A path object with a modified smooth part
            reflecting the subsampling.

        '''
        subsample_loss = self.saturated_loss.subsample(case_idx)
        return self.__class__(subsample_loss,
                              self._X,
                              self.l1_penalty,
                              self.l2_penalty,
                              elastic_net_param=self.elastic_net_param,
                              alpha=self.alpha,
                              l1_alpha=None)

    def check_KKT(self,
                  grad_solution,
                  solution,
                  lagrange,
                  penalty=None):

        '''

        Check KKT conditions over
        the groups in the path.
        Returns boolean indicating
        which groups are failing the KKT conditions
        (these could be `active` groups or
        `inactive` groups).

        Parameters
        ----------

        grad_solution : ndarray
             Candidate for gradient of smooth loss at 
             Lagrange value `lagrange`.

        solution : ndarray
             Candidate for solution to problem 
             Lagrange value `lagrange`.

        lagrange : float
             Lagrange value for penalty

        penalty : object (optional)
             A sparse group block penalty. If None, defaults
             to `self.penalty`.

        '''
        if penalty is None:
            penalty = self.penalty

        active, inactive_ranks = _check_KKT(grad_solution, 
                                            solution, 
                                            self.penalty.l1_weight,
                                            self.penalty.l2_weight,
                                            lagrange)
        return active > 0, inactive_ranks

    def strong_set(self,
                   lagrange_cur,
                   lagrange_new,
                   grad_solution):

        _strong_bool = _strong_set(self.penalty.l1_weight,
                                   self.penalty.l2_weight,
                                   lagrange_cur,
                                   lagrange_new,
                                   grad_solution)
        _strong = np.nonzero(_strong_bool)[0]
        return (_strong, 
                _strong_bool,
                _strong)
                                             
    def solve_subproblem(self, 
                         solution,
                         candidate_groups, 
                         lagrange_new, 
                         **solve_args):
    
        # solve a problem with a candidate set

        sub_loss, sub_penalty, sub_X, candidate_bool = _restricted_problem(self.X, 
                                                                           self.saturated_loss, 
                                                                           self.alpha * lagrange_new, 
                                                                           self.penalty.l1_weight,
                                                                           self.penalty.l2_weight,
                                                                           candidate_groups,
                                                                           self.subsample_columns)
        if self.alpha < 1:
            sub_elastic_net = self.enet_loss(lagrange_new,
                                             candidate_groups)

            sub_loss = smooth_sum([sub_loss, sub_elastic_net])
            sub_loss.shape = sub_elastic_net.shape

        sub_problem = simple_problem(sub_loss, sub_penalty)
        sub_problem.coefs[:] = solution[candidate_bool] # warm start
        sub_soln = sub_problem.solve(**solve_args)
        sub_grad = sub_loss.smooth_objective(sub_soln, mode='grad') 
        sub_linear_pred = sub_X.dot(sub_soln)
        return sub_problem.final_step, sub_grad, sub_soln, sub_linear_pred, candidate_bool

    def updated_ever_active(self,
                            index_obj,
                            group_ids=False): # ignored as result is the same):
        if not hasattr(self, '_ever_active'):
            self._ever_active = np.zeros(self.group_shape, np.bool)
        _ever_active = self._ever_active.copy()
        _ever_active[index_obj] = True
        return list(np.nonzero(_ever_active)[0])

    @property
    def unpenalized(self):
        """
        Unpenalized groups and variables.

        Returns
        -------

        groups : sequence
            Groups with weights equal to 0.

        variables : ndarray
            Boolean indicator that is True if no penalty on that variable.

        """
        return [], []

    def restricted_penalty(self, subset):
        vars = np.zeros(self.penalty.shape[0], np.bool)
        if subset is not None:
            vars[subset] = True
        else:
            vars[:] = True
        return sparse_group_block((vars.sum(), self.shape[1]),
                                  self.penalty.l1_weight,
                                  self.penalty.l2_weight,
                                  lagrange=1)

# Some loss factories with common X

def multinomial(X, Y, *args, **keyword_args):
    Y = np.asarray(Y)
    return sparse_group_common_path(mglm.multinomial_loglike(Y.shape, Y), X, *args, **keyword_args)

def multiresponse_gaussian(X, Y, *args, **keyword_args):
    Y = np.asarray(Y)
    loss = mglm.stacked_common_loglike.gaussian(Y.T)
    return sparse_group_common_path(loss, X, *args, **keyword_args)

class sparse_group_block_path(sparse_group_common_path):

    BIG = 1e12 # lagrange parameter for finding null solution

    """
    Responses here have different Xs but common
    set of columns. Design is transpose of "seemingly unrelated regressions".

    """

    def __init__(self, 
                 saturated_loss, # shape (n,q) -- multiresponse
                 Xs, 
                 l1_penalty,
                 l2_penalty,
                 elastic_net_param=None,
                 alpha=1.,  # elastic net mixing -- 1 is LASSO
                 l1_alpha=None): # this is \alpha of SGL paper

        sparse_group_common_path.__init__(self,
                                          saturated_loss,
                                          block_columns(Xs),
                                          l1_penalty,
                                          l2_penalty,
                                          elastic_net_param=elastic_net_param,
                                          alpha=alpha,
                                          l1_alpha=l1_alpha)

        self._Xs = Xs

    # methods potentially overwritten in subclasses for I/O considerations

    def linpred(self,
                coef,
                X,
                test):
        """
        Form linear predictor for given set of coefficients.

        Typical use case will have coef of shape `(l,p)` for
        univariate response regressions and `(l,q,p)` for
        multiple response regressions (e.g. multinomial with
        `q` classes) where `l` is the number of Lagrange
        parameters.

        Parameters
        ----------

        coef : ndarray
            A set of coefficients. 

        X : object
            Representation of design matrix, usually `self.X` -- subclasses
            may not assume this is actually an array.

        test : index
            Indices of X.dot(coef.T) to return.

        Returns
        -------

        linpred : Matrix of linear predictors.

        """
        coef = np.asarray(coef)
        result = np.array([X.dot(c.T) for c in coef]).T[test]
        return result

    def subsample_columns(self, 
                          X, 
                          columns):
        """
        Extract columns of self.Xs into ndarray or
        Argument `X` is ignored here.
        """
        return block_columns([subsample_columns(Xblock, 
                                                columns)
                              for Xblock in self._Xs])
    
    def subsample(self,
                  case_idx):
        '''

        Create a new path, by subsampling
        cases of `self.saturated_loss`.

        Case weights are computed
        with `self.saturated_loss.subsample`.

        Parameters
        ----------

        case_idx : index
            An index-like object used 
            to specify which cases to include
            in the subsample.

        Returns
        -------

        subsample_path : path object
            A path object with a modified smooth part
            reflecting the subsampling.

        '''
        subsample_loss = self.saturated_loss.subsample(case_idx)
        return self.__class__(subsample_loss,
                              self._Xs,
                              self.l1_penalty,
                              self.l2_penalty,
                              elastic_net_param=self.elastic_net_param,
                              alpha=self.alpha,
                              l1_alpha=None)

def stacked_gaussian(Xs, Ys, *args, **keyword_args):
    loss = glm.stacked_loglike.gaussian(Ys) # Ys a sequence of response vectors
    return sparse_group_block_path(loss, Xs, *args, **keyword_args)

def stacked_cox(Xs, Ts, Cs, *args, **keyword_args):
    loss = glm.stacked_loglike.cox(Ts, Cs) # Ts a sequence of event times,
                                           # Cs a sequence of censor indicators
    return sparse_group_block_path(loss, Xs, *args, **keyword_args)

def stacked_logistic(Xs, Ys, *args, **keyword_args):
    loss = glm.stacked_loglike.logistic(Ys) # Ys a sequence of response vectors
    return sparse_group_block_path(loss, Xs, *args, **keyword_args)

def stacked_probit(Xs, Ys, *args, **keyword_args):
    loss = glm.stacked_loglike.probit(Ys) # Ys a sequence of response vectors
    return sparse_group_block_path(loss, Xs, *args, **keyword_args)

def stacked_cloglog(Xs, Ys, *args, **keyword_args):
    loss = glm.stacked_loglike.cloglog(Ys) # Ys a sequence of response vectors
    return sparse_group_block_path(loss, Xs, *args, **keyword_args)

def stacked_poisson(Xs, Ys, *args, **keyword_args):
    loss = glm.stacked_loglike.poisson(Ys) # Ys a sequence of response vectors
    return sparse_group_block_path(loss, Xs, *args, **keyword_args)

def stacked_huber(Xs, Ys, smoothing_parameter, *args, **keyword_args):
    loss = glm.stacked_loglike.huber(Ys, smoothing_parameter) # Ys a sequence of response vectors
    return sparse_group_block_path(loss, Xs, *args, **keyword_args)

def stacked_huber_svm(Xs, Ys, smoothing_parameter, *args, **keyword_args):
    loss = glm.stacked_loglike.huber_svm(Ys, smoothing_parameter) # Ys a sequence of response vectors
    return sparse_group_block_path(loss, Xs, *args, **keyword_args)

# private functions

def _candidate_bool(groups, candidate_groups):

    candidate_bool = np.zeros(groups.shape, np.bool)
    for g in candidate_groups:
        group = groups == g
        candidate_bool += group

    return candidate_bool

def _restricted_elastic_net(elastic_net_params, 
                            penalized,
                            lagrange, 
                            alpha,
                            candidate_groups):

    new_params = elastic_net_params * (1 - alpha)
    new_params[penalized] *= lagrange 
    if candidate_groups is not None:
        new_params = new_params[candidate_groups]
    return quadratic_loss(new_params.shape,
                          new_params,
                          Qdiag=True)

def _restricted_problem(X, 
                        saturated_loss, 
                        alpha,
                        l1_weight,
                        l2_weight,
                        candidate_groups,
                        subsample_columns):

    X_candidate = subsample_columns(X, candidate_groups)
    candidate_bool = np.zeros(X.input_shape[0], np.bool)
    candidate_bool[candidate_groups] = True
    restricted_penalty = sparse_group_block(X_candidate.input_shape,
                                            l1_weight,
                                            l2_weight,
                                            lagrange=alpha)
    restricted_loss = affine_smooth(saturated_loss, X_candidate)
    restricted_loss.shape = restricted_penalty.shape

    X_c = astransform(X_candidate)
    return restricted_loss, restricted_penalty, X_candidate, candidate_bool

# for paths

def _strong_set(l1_weight,
                l2_weight,
                lagrange_cur, 
                lagrange_new, 
                grad,
                slope_estimate=1):

    """
    Guess at active groups at 
    lagrange_new based on gradient
    at lagrange_cur.

    Return a boolean indicator array.
    """

    thresh = (slope_estimate + 1) * lagrange_new - slope_estimate * lagrange_cur
    test = np.array([_inside_set_strong(grad[i],
                                        thresh,
                                        l1_weight,
                                        l2_weight) for i in range(grad.shape[0])]) == 0
    return test

def _check_KKT(grad, 
               solution, 
               l1_weight,
               l2_weight,
               lagrange, 
               tol=1.e-2):

    """
    Check whether (grad, solution) satisfy
    KKT conditions at a given tolerance.

    Assumes lagrange form of penalty.

    """

    ACTIVE_L1 = 10
    ACTIVE_NORM = 11
    ACTIVE_L2 = 12
    INACTIVE = 2

    norm_soln = np.sqrt(np.sum(solution**2, 1))
    active = norm_soln > tol * max(norm_soln.sum(0), 1)
    active_results = np.zeros(grad.shape[0])
    inactive_results = np.zeros(grad.shape[0])

    for g in np.nonzero(active)[0]:
        subgrad_g = -grad[g]
        soln_g = solution[g]
        val_g, l1subgrad_g, l2subgrad_g = _gauge_function_dual_strong(subgrad_g,
                                                                      l1_weight,
                                                                      l2_weight)
        if val_g < lagrange * (1 - tol):
            active_results[g] = ACTIVE_NORM
        nonz = soln_g != 0

        # nonzero coordinates need the right sign and size
        if (np.linalg.norm((l1subgrad_g - l1_weight * np.sign(soln_g) * lagrange)[nonz]) > 
            tol * max(1, np.linalg.norm(soln_g))):
            active_results[g] = ACTIVE_L1

        # l2 subgrad should be parallel to soln_g
        if np.linalg.norm(l2subgrad_g / np.linalg.norm(l2subgrad_g) - 
                          soln_g / np.linalg.norm(soln_g)) > tol:
            active_results[g] = ACTIVE_L2

    for g in np.nonzero(~active)[0]:
        subgrad_g = -grad[g]
        inactive_results[g] = _gauge_function_dual_strong(subgrad_g, 
                                                          l1_weight,
                                                          l2_weight)[0] / lagrange

    inactive_ranks = inactive_results.shape[0] - np.argsort(inactive_results)
    inactive_ranks[inactive_results <= 1 + tol] = -1

    return active_results, inactive_ranks

